@@ -4,7 +4,6 @@ A2A protocol server implementation for SwissKnife.
 
 import os
 import json
-import logging
 from typing import Callable, Optional
 from starlette.applications import Starlette
 from starlette.responses import JSONResponse
@@ -13,8 +12,9 @@ from starlette.requests import Request
 from starlette.middleware import Middleware
 from sse_starlette.sse import EventSourceResponse
 from pydantic import ValidationError
-from AgentCrew.modules.a2a.common.server import AuthMiddleware
+from .common.server.auth_middleware import AuthMiddleware
 
+from loguru import logger
 from AgentCrew.modules.agents import AgentManager
 from .registry import AgentRegistry
 from .task_manager import MultiAgentTaskManager
@@ -32,8 +32,6 @@ from a2a.types import (
     CancelTaskRequest,
 )
 
-from AgentCrew.modules import logger
-
 
 class A2AServer:
     """A2A server that exposes multiple agents"""
@@ -93,11 +91,7 @@ class A2AServer:
                         "/",
                         self._process_jsonrpc_request_factory(agent_name),
                         methods=["POST"],
-                        middleware=[
-                            Middleware(
-                                AuthMiddleware, api_key=self.api_key, logger=logger
-                            )
-                        ],
+                        middleware=[Middleware(AuthMiddleware, api_key=self.api_key)],
                     ),
                 ],
             )
@@ -139,6 +133,7 @@ class A2AServer:
 
         async def process_jsonrpc_request(request: Request):
             logger.debug(f"Received JSON-RPC request for agent {agent_name}")
+            body = None
             try:
                 # Get task manager for this agent
                 task_manager = self.task_manager.get_task_manager(agent_name)
@@ -156,7 +151,7 @@ class A2AServer:
                 try:
                     json_rpc_request = A2ARequest.model_validate(body)
                 except ValidationError as e:
-                    logging.debug(f"cannot validate_python {e} ")
+                    logger.debug(f"cannot validate_python {e} ")
                     error = JSONRPCErrorResponse(
                         id=body.get("id"),
                         error=InvalidRequestError(data=e.errors()),
@@ -188,7 +183,7 @@ class A2AServer:
                         return JSONResponse(result_stream.model_dump(exclude_none=True))
 
                     async def event_generator():
-                        async for item in result_stream:
+                        async for item in result_stream:  # type: ignore
                             yield {
                                 "data": json.dumps(item.model_dump(exclude_none=True))
                             }
@@ -214,7 +209,7 @@ class A2AServer:
                         return JSONResponse(result_stream.model_dump(exclude_none=True))
 
                     async def event_generator():
-                        async for item in result_stream:
+                        async for item in result_stream:  # type: ignore
                             yield {
                                 "data": json.dumps(item.model_dump(exclude_none=True))
                             }
@@ -236,7 +231,7 @@ class A2AServer:
                 else:
                     logger.error(f"Invalid method requested: {method}")
                     logger.error(f"Request ID: {json_rpc_request.root.id}")
-                    logger.error(f"Request params: {json_rpc_request.root.params}")
+                    logger.error(f"Request params: {json_rpc_request.root.params}")  # type: ignore
                     error = JSONRPCErrorResponse(
                         id=json_rpc_request.root.id,
                         error=JSONRPCError(code=-32601, message="Method not found"),
@@ -269,7 +264,7 @@ class A2AServer:
             except Exception as e:
                 logger.exception(f"Error processing request: {e}")
                 error = JSONRPCErrorResponse(
-                    id=body.get("id") if "body" in locals() else None,
+                    id=body.get("id") if body else None,
                     error=InternalError(),
                 )
                 return JSONResponse(
