@@ -1,6 +1,5 @@
 from .local_agent import LocalAgent
-from .base import BaseAgent
 from .manager import AgentManager
 from .remote_agent import RemoteAgent
 
-__all__ = ["AgentManager", "LocalAgent", "BaseAgent", "RemoteAgent"]
+__all__ = ["AgentManager", "LocalAgent", "RemoteAgent"]
