@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from typing import AsyncGenerator, Tuple, Dict, List, Optional, Any, Callable
+from typing import AsyncGenerator, Tuple, Dict, List, Optional, Any, Callable, Union
 from enum import Enum
 
 
@@ -40,9 +40,14 @@ class BaseAgent(ABC):
         """
         pass
 
+    @abstractmethod
+    def append_message(self, messages: Union[Dict, List[Dict]]):
+        """Append a message or list of messages to the agent's history."""
+        pass
+
     @property
     @abstractmethod
-    def std_history(self) -> List:
+    def clean_history(self) -> List:
         pass
 
     @abstractmethod
@@ -94,4 +99,7 @@ class BaseAgent(ABC):
 
     @abstractmethod
     def get_process_result(self) -> Tuple:
+        """
+        @DEPRECATED: Use the callback in process_messages instead.
+        """
         pass
