@@ -1,11 +1,15 @@
 from datetime import datetime
 import os
 import time
-from typing import Dict, Any, List, Optional, Callable
+import copy
+from typing import Dict, Any, List, Optional, Callable, Literal, Union
 from AgentCrew.modules.llm import BaseLLMService
-from AgentCrew.modules.llm.message import MessageTransformer
-from AgentCrew.modules.agents.base import BaseAgent, MessageType
-from AgentCrew.modules import logger
+
+from .base import BaseAgent, MessageType
+from loguru import logger
+
+SHRINK_CONTEXT_THRESHOLD = 90_000
+SHRINK_LENGTH_THRESHOLD = 15
 
 
 class LocalAgent(BaseAgent):
@@ -20,6 +24,8 @@ class LocalAgent(BaseAgent):
         tools: List[str],
         temperature: Optional[float] = None,
         is_remoting_mode: bool = False,
+        voice_enabled: Literal["full", "partial", "disabled"] = "disabled",
+        voice_id: Optional[str] = None,
     ):
         """
         Initialize a new agent.
@@ -29,10 +35,10 @@ class LocalAgent(BaseAgent):
             description: A description of the agent's capabilities
             llm_service: The LLM service to use for this agent
             services: Dictionary of available services
+            voice_enabled: Whether voice features are enabled for this agent
+            voice_id: Voice ID to use for text-to-speech
         """
         super().__init__(name, description)
-        # self.name = name
-        # self.description = description
         self.llm = llm_service
         self.temperature = temperature
         self.services = services
@@ -40,14 +46,17 @@ class LocalAgent(BaseAgent):
         self.system_prompt = None
         self.custom_system_prompt = None
         self.tool_prompts = []
-        self.is_remoting_mode = is_remoting_mode
-        # self.history = []
-        # self.shared_context_pool: Dict[str, List[int]] = {}
-        # Store tool definitions in the same format as ToolRegistry
+        self.is_remoting_mode: bool = is_remoting_mode
+        self.input_tokens_usage = 0
+        self.output_tokens_usage = 0
+        self.voice_enabled: Literal["full", "partial", "disabled"] = voice_enabled
+        self.voice_id: Optional[str] = voice_id
+
         self.tool_definitions = {}  # {tool_name: (definition_func, handler_factory, service_instance)}
         self.registered_tools = (
             set()
         )  # Set of tool names that are registered with the LLM
+        self._defer_tool_registration = False
         self.mcps_loading = []
 
     def _extract_tool_name(self, tool_def: Any) -> str:
@@ -70,12 +79,22 @@ class LocalAgent(BaseAgent):
         else:
             raise ValueError("Could not extract tool name from definition")
 
+    def append_message(self, messages: Union[Dict, List[Dict]]):
+        copy_messages = copy.deepcopy(messages)
+        if isinstance(copy_messages, List):
+            self.history.extend(copy_messages)
+        else:
+            self.history.append(copy_messages)
+
     def register_tools(self):
         """
         Register tools for this agent using the services dictionary.
         """
 
-        if self.services.get("agent_manager"):
+        if (
+            self.services.get("agent_manager")
+            and self.services["agent_manager"].enforce_transfer
+        ):
             self.tool_prompts.append(
                 self.services["agent_manager"].get_agents_list_prompt()
             )
@@ -93,11 +112,20 @@ class LocalAgent(BaseAgent):
                     register as register_transfer,
                     transfer_tool_prompt,
                 )
+                from AgentCrew.modules.agents.tools.ask import (
+                    register as register_ask,
+                    ask_tool_prompt,
+                )
 
                 register_transfer(self.services["agent_manager"], self)
                 self.tool_prompts.append(
                     transfer_tool_prompt(self.services["agent_manager"])
                 )
+
+                # Register the ask tool (always available)
+                register_ask(self)
+                self.tool_prompts.append(ask_tool_prompt())
+
         for tool_name in self.tools:
             if self.services and tool_name in self.services:
                 service = self.services[tool_name]
@@ -138,6 +166,24 @@ class LocalAgent(BaseAgent):
                         )
 
                         register_image_generation(service, self)
+                    elif tool_name == "browser":
+                        from AgentCrew.modules.browser_automation.tool import (
+                            register as register_browser,
+                        )
+
+                        register_browser(service, self)
+                    elif tool_name == "file_editing":
+                        from AgentCrew.modules.file_editing.tool import (
+                            register as register_file_editing,
+                        )
+
+                        register_file_editing(service, self)
+                    elif tool_name == "command_execution":
+                        from AgentCrew.modules.command_execution.tool import (
+                            register as register_command_execution,
+                        )
+
+                        register_command_execution(service, self)
                     else:
                         logger.warning(f"⚠️ Tool {tool_name} not found in services")
             else:
@@ -182,7 +228,7 @@ class LocalAgent(BaseAgent):
             prompt: The system prompt
         """
         return (
-            prompt.replace("{current_date}", datetime.today().strftime("%A, %d/%m/%Y"))
+            prompt.replace("{current_date}", datetime.today().strftime("%A, %d %B %Y"))
             .replace("{cwd}", os.getcwd())
             .replace("{current_agent_name}", self.name)
             .replace("{current_agent_description}", self.description)
@@ -223,7 +269,7 @@ class LocalAgent(BaseAgent):
 
         # Reinitialize MCP session manager for the current agent
         if not self.is_remoting_mode:
-            from AgentCrew.modules.mcpclient.manager import MCPSessionManager
+            from AgentCrew.modules.mcpclient import MCPSessionManager
 
             mcp_manager = MCPSessionManager.get_instance()
             if mcp_manager.initialized:
@@ -239,9 +285,7 @@ class LocalAgent(BaseAgent):
 
         self.llm.set_system_prompt(self._parse_system_prompt(system_prompt))
         self.llm.temperature = self.temperature if self.temperature is not None else 0.4
-        while len(self.mcps_loading) > 0:
-            time.sleep(0.2)
-        self._register_tools_with_llm()
+        self._defer_tool_registration = True
         self.is_active = True
         return True
 
@@ -262,7 +306,7 @@ class LocalAgent(BaseAgent):
         self.mcps_loading = []
         # Reinitialize MCP session manager for the current agent
         if not self.is_remoting_mode:
-            from AgentCrew.modules.mcpclient.manager import MCPSessionManager
+            from AgentCrew.modules.mcpclient import MCPSessionManager
 
             mcp_manager = MCPSessionManager.get_instance()
             if mcp_manager.initialized:
@@ -313,6 +357,7 @@ class LocalAgent(BaseAgent):
                 self.registered_tools.add(tool_name)
             except Exception as e:
                 logger.error(f"Error registering tool {tool_name}: {e}")
+        self._defer_tool_registration = False
 
     def _clear_tools_from_llm(self):
         """
@@ -324,10 +369,8 @@ class LocalAgent(BaseAgent):
             # Note: We don't clear self.tool_definitions as we want to keep the definitions
 
     @property
-    def std_history(self):
-        return MessageTransformer.standardize_messages(
-            self.history, self.llm.provider_name, self.name
-        )
+    def clean_history(self):
+        return self.history
 
     def get_provider(self) -> str:
         return self.llm.provider_name
@@ -335,20 +378,120 @@ class LocalAgent(BaseAgent):
     def is_streaming(self) -> bool:
         return self.llm.is_stream
 
+    def _format_tool_result(
+        self,
+        tool_use: Dict,
+        tool_result: Any,
+        is_error: bool = False,
+        is_rejected: bool = False,
+    ) -> Dict[str, Any]:
+        """
+        Format a tool result for OpenAI API.
+
+        Args:
+            tool_use: The tool use details
+            tool_result: The result from the tool execution
+            is_error: Whether the result is an error
+
+        Returns:
+            A formatted message for tool response
+        """
+        # OpenAI format for tool responses
+        message = {
+            "role": "tool",
+            "agent": self.name,
+            "tool_call_id": tool_use["id"],
+            "tool_name": tool_use["name"],
+            "content": tool_result,
+        }
+
+        # Add error indication if needed
+        if is_error:
+            message["content"] = f"ERROR: {str(message['content'])}"
+        if is_rejected:
+            message["is_rejected"] = True
+
+        return message
+
+    def _format_assistant_message(
+        self, assistant_response: str, tool_uses: list[Dict] | None = None
+    ) -> Dict[str, Any]:
+        """
+        Format the assistant's response into the appropriate message format for the LLM provider.
+
+        Args:
+            assistant_response (str): The text response from the assistant
+            tool_use (Dict, optional): Tool use information if a tool was used
+
+        Returns:
+            Dict[str, Any]: A properly formatted message to append to the messages list
+        """
+        if tool_uses and any(tu.get("id") for tu in tool_uses):
+            return {
+                "role": "assistant",
+                "agent": self.name,
+                "content": assistant_response,
+                "tool_calls": [
+                    {
+                        "id": tool_use["id"],
+                        "name": tool_use["name"],
+                        "arguments": tool_use["input"],
+                        "type": tool_use.get("type", "tool_call"),
+                    }
+                    for tool_use in tool_uses
+                    if tool_use.get("id")  # Only include tool calls with valid IDs
+                ],
+            }
+        else:
+            return {
+                "agent": self.name,
+                "role": "assistant",
+                "content": assistant_response,
+            }
+
+    def _format_thinking_message(self, thinking_data) -> Optional[Dict[str, Any]]:
+        """
+        Format thinking content into the appropriate message format for Claude.
+
+        Args:
+            thinking_data: Tuple containing (thinking_content, thinking_signature)
+                or None if no thinking data is available
+
+        Returns:
+            Dict[str, Any]: A properly formatted message containing thinking blocks
+        """
+        if not thinking_data:
+            return None
+
+        thinking_content, thinking_signature = thinking_data
+
+        if not thinking_content:
+            return None
+
+        # For Claude, thinking blocks need to be preserved in the assistant's message
+        thinking_block = {"type": "thinking", "thinking": thinking_content}
+
+        # Add signature if available
+        if thinking_signature:
+            thinking_block["signature"] = thinking_signature
+
+        return {"role": "assistant", "agent": self.name, "content": [thinking_block]}
+
     def format_message(
         self, message_type: MessageType, message_data: Dict[str, Any]
     ) -> Optional[Dict[str, Any]]:
         if message_type == MessageType.Assistant:
-            return self.llm.format_assistant_message(
+            return self._format_assistant_message(
                 message_data.get("message", ""), message_data.get("tool_uses", None)
             )
         elif message_type == MessageType.Thinking:
-            return self.llm.format_thinking_message(message_data.get("thinking", None))
+            return self._format_thinking_message(message_data.get("thinking", None))
         elif message_type == MessageType.ToolResult:
-            return self.llm.format_tool_result(
+            return self._format_tool_result(
                 message_data.get("tool_use", {}),
                 message_data.get("tool_result", ""),
                 message_data.get("is_error", False),
+                message_data.get("is_rejected", False),
             )
         elif message_type == MessageType.FileContent:
             return self.llm.process_file_for_message(message_data.get("file_uri", ""))
@@ -381,20 +524,6 @@ class LocalAgent(BaseAgent):
         if was_active:
             self.deactivate()
 
-        # Get the current provider
-        current_provider = self.llm.provider_name
-
-        # If we're switching providers, convert messages
-        if current_provider != new_llm_service.provider_name:
-            # Standardize messages from current provider
-            std_messages = MessageTransformer.standardize_messages(
-                self.history, current_provider, self.name
-            )
-            # Convert to new provider format
-            self.history = MessageTransformer.convert_messages(
-                std_messages, new_llm_service.provider_name
-            )
-
         # Update the LLM service
         self.llm = new_llm_service
 
@@ -404,6 +533,213 @@ class LocalAgent(BaseAgent):
 
         return True
 
+    def _build_adaptive_behavior_context(self) -> Dict[str, Any]:
+        from AgentCrew.modules.memory.context_persistent import (
+            ContextPersistenceService,
+        )
+
+        adaptive_messages = {
+            "role": "user",
+            "content": [],
+        }
+        if "context_persistent" not in self.services or not isinstance(
+            self.services["context_persistent"], ContextPersistenceService
+        ):
+            return adaptive_messages
+        adaptive_behaviors = self.services["context_persistent"].get_adaptive_behaviors(
+            self.name
+        )
+        if (
+            self.services.get("agent_manager")
+            and self.services["agent_manager"].one_turn_process
+        ):
+            adaptive_messages["content"].append(
+                {
+                    "type": "text",
+                    "text": """My next request is single-turn conversation.
+You must analyze then execute it with your available tools and give answer without asking for confirmation or clarification.""",
+                }
+            )
+
+        if len(adaptive_behaviors.keys()) > 0:
+            adaptive_text = "  \n".join(
+                [
+                    f"<Behavior id='{key}'>{value}</Behavior>"
+                    for key, value in adaptive_behaviors.items()
+                ]
+            )
+            adaptive_messages["content"].append(
+                {
+                    "type": "text",
+                    "text": f"""# MANDATORY: APPLY list of <Adaptive_Behaviors> before responding. 
+If `when` conditions in <Behavior> match, update your responses with behaviors immediately—they override default instruction.
+<Adaptive_Behaviors>
+{adaptive_text}
+</Adaptive_Behaviors>""",
+                }
+            )
+        return adaptive_messages
+
+    def _enhance_agent_context_messages(self, final_messages: List[Dict[str, Any]]):
+        last_user_index = next(
+            (
+                i
+                for i, msg in enumerate(reversed(final_messages))
+                if msg.get("role") == "user"
+            ),
+            None,
+        )
+        if last_user_index is None:
+            return
+        last_user_index = len(final_messages) - 1 - last_user_index
+        adaptive_messages = self._build_adaptive_behavior_context()
+        if (
+            len(final_messages[last_user_index].get("content", [])) > 0
+            and final_messages[last_user_index]["content"][0]
+            .get("text", "")
+            .find("<Transfer_Tool>")
+            != 0
+        ):
+            if (
+                self.services.get("agent_manager")
+                and self.services["agent_manager"].enforce_transfer
+            ):
+                adaptive_messages["content"].insert(
+                    0,
+                    {
+                        "type": "text",
+                        "text": """Before processing my request:
+    - Break my request into sub-tasks when applicable.
+    - For each sub-task, evaluate other agents capabilities.
+    - Transfer sub-task to other agent if they are more suitable. 
+    - Keep the evaluating quick and concise using xml format within <agent_evaluation> tags.
+    - Skip agent evaluation if user request is when...,[action]... related to adaptive behaviors call `adapt` tool instead.""",
+                    },
+                )
+        if len(adaptive_messages["content"]) > 0:
+            final_messages.insert(last_user_index, adaptive_messages)
+
+        agent_manager = self.services.get("agent_manager", None)
+        if agent_manager and agent_manager.defered_transfer:
+            last_assistant_index = next(
+                (
+                    i
+                    for i, msg in enumerate(reversed(final_messages))
+                    if msg.get("role") == "assistant"
+                ),
+                -1,
+            )
+            final_messages.insert(
+                last_assistant_index,
+                {
+                    "role": "user",
+                    "content": [
+                        {
+                            "type": "text",
+                            "text": f"""<Transfer_Reminder>Make sure to transfer after the task is completed: {agent_manager.defered_transfer}</Transfer_Reminder>""",
+                        }
+                    ],
+                },
+            )
+
+    def _clean_shrinkable_tool_result(self, final_messages: List[Dict[str, Any]]):
+        """
+        Clean unique tool results by replacing all but the last [UNIQUE] tool result with "[INVALIDATED]".
+
+        Args:
+            final_messages: List of message dictionaries to process
+        """
+        # Find all indices of tool messages that start with [UNIQUE]
+        unique_tool_indices = []
+        agent_manager = self.services.get("agent_manager", None)
+
+        is_shrinkable = (
+            agent_manager.context_shrink_enabled if agent_manager else False
+        ) and self.input_tokens_usage > SHRINK_CONTEXT_THRESHOLD
+        shrink_threshold = len(final_messages) - SHRINK_LENGTH_THRESHOLD
+        shrink_excluded = (
+            set(agent_manager.shrink_excluded_list) if agent_manager else []
+        )
+
+        for i, msg in enumerate(final_messages):
+            # Check different message formats for tool results
+            content = None
+
+            if msg.get("role") == "assistant":
+                if len(msg.get("tool_calls", [])) == 0:
+                    continue
+
+                if is_shrinkable and i < shrink_threshold:
+                    for tool_call in msg.get("tool_calls", []):
+                        if tool_call.get("name") in shrink_excluded:
+                            continue
+                        tool_call["arguments"] = {}
+
+            elif msg.get("role") == "tool":
+                tool_name = msg.get("tool_name", "")
+
+                # Remove denied tools after agent correct it
+                if msg.get("is_rejected", False):
+                    has_last_user_message = next(
+                        (True for _ in final_messages[i:] if msg.get("role") == "user"),
+                        False,
+                    )
+                    if has_last_user_message:
+                        tool_id = msg.get("tool_call_id", "")
+                        last_assistant_msg = final_messages[i - 1]
+                        for tool_call in last_assistant_msg.get("tool_calls", []):
+                            if tool_call.get("id", "") == tool_id:
+                                tool_call["arguments"] = {}
+                                break
+
+                if tool_name in shrink_excluded:
+                    continue
+
+                if is_shrinkable and i < shrink_threshold:
+                    msg["content"] = "[REDACTED]"
+                    continue
+
+                # Check if content starts with [UNIQUE]
+                content = msg.get("content", "")
+                if (
+                    content
+                    and isinstance(content, str)
+                    and content.startswith("[UNIQUE]")
+                ):
+                    unique_tool_indices.append(i)
+                elif content and isinstance(content, list):
+                    if (
+                        len(
+                            [
+                                d.get("text", "")
+                                for d in content
+                                if isinstance(d, dict)
+                                and d.get("text", "").startswith("[UNIQUE]")
+                            ]
+                        )
+                        > 0
+                    ):
+                        unique_tool_indices.append(i)
+
+        # Replace all but the last [UNIQUE] tool result with "[INVALIDATED]"
+        if len(unique_tool_indices) > 1:
+            for i in unique_tool_indices[:-1]:  # All except the last one
+                msg = final_messages[i]
+
+                # Update content based on message format
+                if msg.get("role") == "tool" and "content" in msg:
+                    msg["content"] = "[INVALIDATED]"
+
+                elif msg.get("role") == "user" and isinstance(msg.get("content"), list):
+                    for content_item in msg["content"]:
+                        if (
+                            isinstance(content_item, dict)
+                            and content_item.get("type") == "tool_result"
+                            and "content" in content_item
+                        ):
+                            content_item["content"] = "[INVALIDATED]"
+                            break
+
     async def process_messages(
         self,
         messages: Optional[List[Dict[str, Any]]] = None,
@@ -418,93 +754,25 @@ class LocalAgent(BaseAgent):
         Returns:
             The processed messages with the agent's response
         """
-        from AgentCrew.modules.memory.context_persistent import (
-            ContextPersistenceService,
-        )
+
+        if self._defer_tool_registration:
+            while len(self.mcps_loading) > 0:
+                time.sleep(0.2)
+            self._register_tools_with_llm()
 
         assistant_response = ""
         _tool_uses = []
         _input_tokens_usage = 0
         _output_tokens_usage = 0
         # Ensure the first message is a system message with the agent's prompt
-        if not messages:
-            final_messages = list(self.history)
-        else:
-            final_messages = list(messages)
-        if "context_persistent" in self.services and isinstance(
-            self.services["context_persistent"], ContextPersistenceService
-        ):
-            adaptive_behaviors = self.services[
-                "context_persistent"
-            ].get_adaptive_behaviors(self.name)
-            # adaptive behaviors are only added if the last message is from the user
-            if isinstance(final_messages[-1]["content"], str) or (
-                isinstance(final_messages[-1]["content"], list)
-                and final_messages[-1]["content"][0].get("type") != "tool_result"
-            ):
-                adaptive_messages = {
-                    "role": "user",
-                    "content": [],
-                }
-                if (
-                    self.services.get("agent_manager")
-                    and self.services["agent_manager"].enforce_transfer
-                ):
-                    adaptive_messages["content"].append(
-                        {
-                            "type": "text",
-                            "text": """Before processing my request:
-- Skip agent evaluation if user request related to when...do... adaptive behaviors.
-- Break my request into sub-tasks when applicable.
-- For each sub-task, evaluate other agents capabilities.
-- Transfer sub-task to other agent if they are more suitable. 
-- Keep the evaluating quick and concise using xml format within <agent_evaluation> tags.""",
-                        }
-                    )
-                if (
-                    self.services.get("agent_manager")
-                    and self.services["agent_manager"].one_turn_process
-                ):
-                    adaptive_messages["content"].append(
-                        {
-                            "type": "text",
-                            "text": """My next request is one-turn conversation.
-You must analyze process it with your available tools and give answer without asking for confirmation or clarification.""",
-                        }
-                    )
-
-                if len(adaptive_behaviors.keys()) > 0:
-                    adaptive_text = ""
-                    for key, value in adaptive_behaviors.items():
-                        adaptive_text += f"- {value} (id:{key})\n"
-                    adaptive_messages["content"].append(
-                        {
-                            "type": "text",
-                            "text": f"""# MANDATORY: APPLY list of adaptive behaviors before responding. When "when...do..." conditions match, modify your behaviors immediately—they override default logic.
-Ask for clarification if uncertain which behaviors apply.
-## List of adaptive behaviors:
-{adaptive_text}
-END OF ADAPTABLE BEHAVIORS.""",
-                        }
-                    )
-                last_user_index = -1
-                for i, msg in reversed(list(enumerate(final_messages))):
-                    if msg.get("role", "assistant") == "user":
-                        last_user_index = i
-                        break
-                if (
-                    len(final_messages[last_user_index].get("content", [])) > 0
-                    and final_messages[last_user_index]["content"][0]
-                    .get("text", "")
-                    .find("<Transfer_Tool>")
-                    == 0
-                ):
-                    adaptive_messages["content"].pop(0)
-                if len(adaptive_messages["content"]) > 0:
-                    final_messages.insert(last_user_index, adaptive_messages)
+        final_messages = messages[:] if messages else self.history[:]
+        self._enhance_agent_context_messages(final_messages)
+        self._clean_shrinkable_tool_result(final_messages)
         try:
             async with await self.llm.stream_assistant_response(
-                final_messages
+                copy.deepcopy(
+                    final_messages
+                )  # This will prevent llm converting message break the original format
             ) as stream:
                 async for chunk in stream:
                     # Process the chunk using the LLM service
@@ -518,29 +786,6 @@ END OF ADAPTABLE BEHAVIORS.""",
                     ) = self.llm.process_stream_chunk(
                         chunk, assistant_response, _tool_uses
                     )
-                    # if (
-                    #     "<agent_evaluation>" in assistant_response
-                    #     and "</agent_evaluation>" not in assistant_response
-                    # ):
-                    #     continue
-                    # transfer_index_start = assistant_response.find("<agent_evaluation>")
-                    # transfer_index_end = assistant_response.find("</agent_evaluation>")
-                    # if transfer_index_start >= 0 and transfer_index_end >= 0:
-                    #     assistant_response = (
-                    #         assistant_response[:transfer_index_start]
-                    #         + assistant_response[transfer_index_end + 19 :]
-                    #     )
-                    # if chunk_text:
-                    #     chunk_transfer_index_start = chunk_text.find(
-                    #         "<agent_evaluation>"
-                    #     )
-                    #     if chunk_transfer_index_start >= 0:
-                    #         chunk_text = chunk_text[:chunk_transfer_index_start]
-                    #     chunk_transfer_index_end = chunk_text.find(
-                    #         "</agent_evaluation>"
-                    #     )
-                    #     if chunk_transfer_index_end >= 0:
-                    #         chunk_text = chunk_text[chunk_transfer_index_end + 19 :]
                     yield (assistant_response, chunk_text, thinking_chunk)
 
                     if tool_uses:
@@ -549,12 +794,13 @@ END OF ADAPTABLE BEHAVIORS.""",
                         _input_tokens_usage = chunk_input_tokens
                     if chunk_output_tokens > 0:
                         _output_tokens_usage = chunk_output_tokens
+
+            self.input_tokens_usage = _input_tokens_usage
+            self.output_tokens_usage = _output_tokens_usage
             if callback:
                 callback(_tool_uses, _input_tokens_usage, _output_tokens_usage)
             else:
                 self.tool_uses = _tool_uses
-                self.input_tokens_usage = _input_tokens_usage
-                self.output_tokens_usage = _output_tokens_usage
 
         except GeneratorExit as e:
             logger.warning(f"Stream processing interrupted: {e}")
@@ -564,4 +810,7 @@ END OF ADAPTABLE BEHAVIORS.""",
             raise e
 
     def get_process_result(self):
+        """
+        @DEPRECATED: Use the callback in process_messages instead.
+        """
         return (self.tool_uses, self.input_tokens_usage, self.output_tokens_usage)
