@@ -2,8 +2,8 @@ import toml
 import json
 from typing import Dict, Any, Optional, List
 
-from AgentCrew.modules.agents import BaseAgent, LocalAgent
-from AgentCrew.modules.llm.message import MessageTransformer
+from .base import BaseAgent
+from .local_agent import LocalAgent
 
 
 class AgentManager:
@@ -62,8 +62,11 @@ class AgentManager:
         if not self._initialized:
             self.agents: Dict[str, BaseAgent] = {}
             self.current_agent: Optional[BaseAgent] = None
-            self.enforce_transfer: bool = True  # Flag to enforce agent transfers
-            self.one_turn_process: bool = False  # Flag for one-turn processing
+            self.enforce_transfer: bool = True
+            self.one_turn_process: bool = False
+            self.context_shrink_enabled: bool = True
+            self.shrink_excluded_list: List[str] = []
+            self._defered_transfer: str = ""
             self._initialized = True
 
     @classmethod
@@ -102,22 +105,13 @@ class AgentManager:
             True if the agent was selected, False otherwise
         """
         if agent_name in self.agents:
-            # Get the new agent
             new_agent = self.agents[agent_name]
 
-            # If there was a previous agent, deactivate it
             if self.current_agent:
                 self.current_agent.deactivate()
 
-            # Set the new agent as current
             self.current_agent = new_agent
 
-            # if self.current_agent and isinstance(self.current_agent, LocalAgent):
-            #     if not self.current_agent.custom_system_prompt:
-            #         self.current_agent.set_custom_system_prompt(
-            #             self.get_transfer_system_prompt()
-            #         )
-            # Activate the new agent
             if self.current_agent:
                 self.current_agent.activate()
 
@@ -143,6 +137,14 @@ class AgentManager:
         else:
             return None
 
+    @property
+    def defered_transfer(self):
+        return self._defered_transfer
+
+    @defered_transfer.setter
+    def defered_transfer(self, value: str):
+        self._defered_transfer = value
+
     def clean_agents_messages(self):
         for _, agent in self.agents.items():
             agent.history = []
@@ -185,10 +187,7 @@ class AgentManager:
             ]
 
             if agent_messages:
-                agent.history = MessageTransformer.convert_messages(
-                    agent_messages,
-                    agent.get_provider(),
-                )
+                agent.append_message(agent_messages)
 
     def get_current_agent(self) -> BaseAgent:
         """
@@ -213,6 +212,7 @@ class AgentManager:
         Returns:
             A dictionary with the result of the transfer
         """
+        self._defered_transfer = ""
         if target_agent_name not in self.agents:
             raise ValueError(
                 f"Agent '{target_agent_name}' not found. Available_agents: {list(self.agents.keys())}"
@@ -226,7 +226,7 @@ class AgentManager:
         if source_agent:
             if target_agent_name not in source_agent.shared_context_pool:
                 source_agent.shared_context_pool[target_agent_name] = []
-            for i, msg in enumerate(source_agent.std_history):
+            for i, msg in enumerate(source_agent.history):
                 if i not in source_agent.shared_context_pool[target_agent_name]:
                     if "content" in msg:
                         content = ""
@@ -280,12 +280,7 @@ class AgentManager:
         self.select_agent(target_agent_name)
         if direct_injected_messages and self.current_agent:
             length_of_current_agent_history = len(self.current_agent.history)
-            self.current_agent.history.extend(
-                MessageTransformer.convert_messages(
-                    direct_injected_messages, self.current_agent.get_provider()
-                )
-            )
-            ## injected messages should not be transfered back to source agent
+            self.current_agent.history.extend(direct_injected_messages)
             if source_agent_name and self.current_agent:
                 if source_agent_name not in self.current_agent.shared_context_pool:
                     self.current_agent.shared_context_pool[source_agent_name] = []
@@ -452,7 +447,7 @@ When system access is requested:
          • Maintain your role as the conversation manager
 
       3. **AGENT_SELECTION:**
-         • Choose the single most appropriate specialist from Available_Agents_List
+         • Choose the single most appropriate specialist from Transferable_Agents
          • Match task requirements to agent capabilities precisely
          • Consider the specialist's tools and expertise domain
 
@@ -464,7 +459,7 @@ When system access is requested:
     <Tool_Usage>
       Required parameters for `delegate` tool:
       • `from_agent`: Your agent name (identifies the delegating agent)
-      • `target_agent`: Exact agent name from Available_Agents_List
+      • `target_agent`: Exact agent name from Transferable_Agents
       • `task_description`: Clear, actionable task with specific objectives
     </Tool_Usage>
   </Delegation_Protocol>
@@ -482,7 +477,7 @@ When system access is requested:
     - You are a specialized agent operating within a multi-agent system
     - MANDATORY: Before any response, perform a systematic evaluation of all available agents
     - Analyze the user's message for domain keywords, technical terms, and task indicators
-    - Cross-reference these against each Ready_To_Transfer_Agents > agent > description and capabilities
+    - Cross-reference these against each Transferable_Agents > agent > description and capabilities
     - When a more appropriate specialist exists, immediately transfer the task using the `transfer` tool
     - Craft precise, actionable task descriptions that enable the target agent to execute effectively without requiring additional clarification
   </Instruction>
@@ -497,7 +492,7 @@ When system access is requested:
     </Step_1_Keyword_Extraction>
 
     <Step_2_Agent_Matching>
-      For each Ready_To_Transfer_Agent:
+      For each Transferable_Agents:
       • Compare extracted keywords against name and description tags
       • Identify direct matches in name or description tags
       • Identify semantic matches in name or description tags
@@ -535,6 +530,7 @@ When system access is requested:
          • Specify any constraints, preferences, or requirements
          • Reference triggering keywords that prompted the transfer
          • Include all relevant context from the user's original message
+         • Tool/Function call results will be omitted during trasnfer. Make sure you always have a compreshensive summary about tool/function call results when transfering
 
       3. **PRE-TRANSFER COMMUNICATION:**
          • Explain to the user why transfer is necessary
@@ -565,7 +561,7 @@ When system access is requested:
 
     <Tool_Usage>
       Required parameters for `transfer` tool:
-      • `target_agent`: Exact agent name from Available_Agents_List
+      • `target_agent`: Exact agent name from Transferable_Agents
       • `task_description`: Action-oriented, specific task with clear objectives and full context
       • `post_action`: (Optional) Next step after task completion
     </Tool_Usage>
