@@ -1,15 +1,20 @@
+from __future__ import annotations
 from dataclasses import dataclass
-from typing import Optional, Tuple, List, Dict
+from typing import Optional, Tuple, List, Dict, Any
 import os
 
-from AgentCrew.modules.agents.local_agent import LocalAgent
 from AgentCrew.modules.chat.file_handler import FileHandler
 from AgentCrew.modules.llm.model_registry import ModelRegistry
 from AgentCrew.modules.llm.service_manager import ServiceManager
 from AgentCrew.modules.chat.consolidation import ConversationConsolidator
 from AgentCrew.modules.config import ConfigManagement
-from AgentCrew.modules.mcpclient import MCPService
+from AgentCrew.modules.agents.local_agent import LocalAgent
 import shlex
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from AgentCrew.modules.mcpclient import MCPService
+    from AgentCrew.modules.chat.message import MessageHandler
 
 
 @dataclass
@@ -25,11 +30,8 @@ class CommandResult:
 class CommandProcessor:
     """Handles command processing for the message handler."""
 
-    def __init__(self, message_handler):
-        from AgentCrew.modules.chat.message import MessageHandler
-
-        if isinstance(message_handler, MessageHandler):
-            self.message_handler = message_handler
+    def __init__(self, message_handler: MessageHandler):
+        self.message_handler = message_handler
 
     async def process_command(self, user_input: str) -> CommandResult:
         """Process a command and return the result."""
@@ -46,7 +48,7 @@ class CommandProcessor:
             return CommandResult(handled=True, clear_flag=True)
         elif user_input.lower() == "/debug":
             self.message_handler._notify(
-                "debug_requested", self.message_handler.agent.history
+                "debug_requested", self.message_handler.agent.clean_history
             )
             self.message_handler._notify(
                 "debug_requested", self.message_handler.streamline_messages
@@ -298,6 +300,12 @@ class CommandProcessor:
 
             selected_turn = self.message_handler.conversation_turns[turn_number - 1]
 
+            selected_message = self.message_handler.streamline_messages[
+                selected_turn.message_index
+            ]
+
+            selected_message_agent = selected_message.get("agent", "")
+
             self.message_handler.streamline_messages = (
                 self.message_handler.streamline_messages[: selected_turn.message_index]
             )
@@ -314,6 +322,8 @@ class CommandProcessor:
             )
             if last_message and last_message.get("agent", ""):
                 self._handle_agent_command(f"/agent {last_message['agent']}")
+            elif selected_message_agent:
+                self._handle_agent_command(f"/agent {selected_message_agent}")
 
             self.message_handler.agent_manager.rebuild_agents_messages(
                 self.message_handler.streamline_messages
@@ -322,7 +332,7 @@ class CommandProcessor:
                 self.message_handler.conversation_turns[: turn_number - 1]
             )
             self.message_handler.last_assisstant_response_idx = len(
-                self.message_handler.agent.history
+                self.message_handler.streamline_messages
             )
 
             self.message_handler._notify(
@@ -514,7 +524,11 @@ class CommandProcessor:
         # Add all successfully processed file contents to messages
         if all_file_contents:
             self.message_handler._messages_append(
-                {"role": "user", "content": all_file_contents}
+                {
+                    "role": "user",
+                    "agent": self.message_handler.agent.name,
+                    "content": all_file_contents,
+                }
             )
 
             # Notify about overall processing results
@@ -594,8 +608,16 @@ class CommandProcessor:
                 )
                 return CommandResult(handled=True, clear_flag=True)
 
+            async def submit_active_voice(audio_data: Any, sample_rate: int):
+                if getattr(self.message_handler, "stream_generator", None):
+                    return
+                transcript = await self._voice_transcript(audio_data, sample_rate)
+                self.message_handler._notify("voice_activate", transcript)
+
             # Start recording
-            result = self.message_handler.voice_service.start_voice_recording()
+            result = self.message_handler.voice_service.start_voice_recording(
+                voice_completed_cb=submit_active_voice
+            )
 
             if result["success"]:
                 self.message_handler._notify("voice_recording_started", None)
@@ -612,37 +634,12 @@ class CommandProcessor:
             self.message_handler._notify("error", f"Voice command failed: {str(e)}")
             return CommandResult(handled=True, clear_flag=True)
 
-    async def _handle_end_voice_command(self, command: str) -> CommandResult:
-        """Handle /end_voice command to stop recording and transcribe."""
-        try:
-            # Check if voice service exists and is recording
-            if self.message_handler.voice_service is None:
-                self.message_handler._notify(
-                    "error",
-                    "No voice service initialized. Use /voice to start recording.",
-                )
-                return CommandResult(handled=True, clear_flag=True)
-
-            if not self.message_handler.voice_service.is_recording():
-                self.message_handler._notify("error", "No recording in progress.")
-                return CommandResult(handled=True, clear_flag=True)
-
-            # Stop recording
-            self.message_handler._notify("voice_recording_stopping", None)
-            stop_result = self.message_handler.voice_service.stop_voice_recording()
-
-            if not stop_result["success"]:
-                self.message_handler._notify("error", stop_result["error"])
-                return CommandResult(handled=True, clear_flag=True)
-
-            # Transcribe
-            self.message_handler._notify("system_message", "🔄 Transcribing audio...")
-
+    async def _voice_transcript(self, audio_data: Any, sample_rate: int):
+        transcribed_text = None
+        if audio_data is not None and self.message_handler.voice_service:
             transcribe_result = await self.message_handler.voice_service.speech_to_text(
-                stop_result["audio_data"], stop_result["sample_rate"]
+                audio_data, sample_rate
             )
-            transcribed_text = None
-
             if transcribe_result["success"]:
                 transcribed_text = transcribe_result["text"]
                 confidence = transcribe_result.get("confidence", 1.0)
@@ -653,19 +650,43 @@ class CommandProcessor:
                     f"✅ Transcribed (confidence: {confidence:.0%}): {transcribed_text}",
                 )
 
-                # Process as regular user input if there's text
-                if transcribed_text.strip():
-                    await self.message_handler.process_user_input(transcribed_text)
-                else:
-                    self.message_handler._notify(
-                        "system_message", "No speech detected in the recording."
-                    )
-
             else:
                 self.message_handler._notify("error", transcribe_result["error"])
 
-            self.message_handler._notify("voice_recording_completed", transcribed_text)
-            return CommandResult(handled=True, clear_flag=False)
+            return transcribed_text
+
+    async def _handle_end_voice_command(self, command: str) -> CommandResult:
+        """Handle /end_voice command to stop recording and transcribe."""
+        try:
+            # Check if voice service exists and is recording
+            if self.message_handler.voice_service is None:
+                # self.message_handler._notify(
+                #     "error",
+                #     "No voice service initialized. Use /voice to start recording.",
+                # )
+                return CommandResult(handled=True, clear_flag=True)
+
+            if not self.message_handler.voice_service.is_recording():
+                # self.message_handler._notify("error", "No recording in progress.")
+                return CommandResult(handled=True, clear_flag=True)
+
+            # Stop recording
+            self.message_handler._notify("voice_recording_stopping", None)
+            stop_result = self.message_handler.voice_service.stop_voice_recording()
+
+            if not stop_result["success"]:
+                # self.message_handler._notify("error", stop_result["error"])
+                return CommandResult(handled=True, clear_flag=True)
+
+            # Transcribe
+            # self.message_handler._notify("system_message", "🔄 Transcribing audio...")
+
+            # transcribed_text = await self._voice_transcript(
+            #     stop_result["audio_data"], stop_result["sample_rate"]
+            # )
+
+            self.message_handler._notify("voice_recording_completed", None)
+            return CommandResult(handled=True, clear_flag=True)
 
         except Exception as e:
             self.message_handler._notify("error", f"End voice command failed: {str(e)}")
@@ -680,11 +701,16 @@ class CommandProcessor:
             self.message_handler.agent_manager.enforce_transfer = not current_state
             new_state = self.message_handler.agent_manager.enforce_transfer
 
+            self.message_handler.agent.deactivate()
+
+            self.message_handler.agent.activate()
+
             # Notify user about the state change
             status = "enabled" if new_state else "disabled"
             self.message_handler._notify(
                 "system_message", f"🔄 Transfer enforcement is now {status}."
             )
+            self.message_handler._notify("transfer_enforce_toggled", status)
 
             return CommandResult(handled=True, clear_flag=True)
 
