@@ -1,18 +1,21 @@
+from __future__ import annotations
 from typing import List, Dict, Any, Optional
 
-from AgentCrew.modules import logger
+from loguru import logger
 from AgentCrew.modules.chat.history import ConversationTurn
 from AgentCrew.modules.agents import RemoteAgent
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from AgentCrew.modules.chat.message import MessageHandler
+
 
 class ConversationManager:
     """Manages conversation state and operations."""
 
-    def __init__(self, message_handler):
-        from AgentCrew.modules.chat.message import MessageHandler
-
-        if isinstance(message_handler, MessageHandler):
-            self.message_handler = message_handler
+    def __init__(self, message_handler: MessageHandler):
+        self.message_handler = message_handler
 
     def start_new_conversation(self):
         """Starts a new persistent conversation, clears history, and gets a new ID."""
@@ -86,6 +89,14 @@ class ConversationManager:
                 conversation_id
             )
             if history:
+                # Backward compatibility: Convert tool messages
+                for msg in history:
+                    if msg.get("role", "user") == "tool":
+                        tool_result = msg.pop("tool_result", None)
+                        if tool_result:
+                            msg["content"] = tool_result.get("content", "")
+                            msg["tool_call_id"] = tool_result.get("tool_use_id", "")
+
                 self.message_handler.current_conversation_id = conversation_id
                 self.message_handler.memory_service.session_id = (
                     self.message_handler.current_conversation_id
@@ -105,7 +116,7 @@ class ConversationManager:
                 )
 
                 self.message_handler.last_assisstant_response_idx = len(
-                    self.message_handler.agent.history
+                    self.message_handler.streamline_messages
                 )
                 try:
                     self.message_handler.latest_assistant_response = (
