@@ -4,25 +4,22 @@ import shlex
 import traceback
 import time
 
-from AgentCrew.modules import logger
+from loguru import logger
 from AgentCrew.modules.agents.base import MessageType
 from AgentCrew.modules.chat.history import ChatHistoryManager
 from AgentCrew.modules.agents import AgentManager
 from AgentCrew.modules.chat.file_handler import FileHandler
-from AgentCrew.modules.llm.message import MessageTransformer
+
 from AgentCrew.modules.config import ConfigManagement
 from AgentCrew.modules.memory import (
     BaseMemoryService,
     ContextPersistenceService,
 )
 from AgentCrew.modules.mcpclient import MCPSessionManager
-
-from AgentCrew.modules.chat.message import (
-    CommandProcessor,
-    ToolManager,
-    ConversationManager,
-    Observable,
-)
+from .command_processor import CommandProcessor
+from .tool_manager import ToolManager
+from .conversation import ConversationManager
+from .base import Observable
 
 
 class MessageHandler(Observable):
@@ -36,6 +33,7 @@ class MessageHandler(Observable):
         self,
         memory_service: BaseMemoryService,
         context_persistent_service: ContextPersistenceService,
+        with_voice: bool = False,
     ):
         """
         Initializes the MessageHandler.
@@ -75,25 +73,25 @@ class MessageHandler(Observable):
         # Check if voice service is available
         from AgentCrew.modules.voice import AUDIO_AVAILABLE
 
-        if AUDIO_AVAILABLE:
-            from AgentCrew.modules.voice import (
-                DeepInfraVoiceService,
-                ElevenLabsVoiceService,
-            )
-
+        if AUDIO_AVAILABLE and with_voice:
             if os.getenv("ELEVENLABS_API_KEY"):
+                from AgentCrew.modules.voice.elevenlabs_service import (
+                    ElevenLabsVoiceService,
+                )
+
                 self.voice_service = ElevenLabsVoiceService()
             elif os.getenv("DEEPINFRA_API_KEY"):
+                from AgentCrew.modules.voice.deepinfra_service import (
+                    DeepInfraVoiceService,
+                )
+
                 self.voice_service = DeepInfraVoiceService()
 
     def _messages_append(self, message):
         """Append a message to the agent history and streamline messages."""
-        self.agent.history.append(message)
+        self.streamline_messages.append(message)
 
-        std_msg = MessageTransformer.standardize_messages(
-            [message], self.agent.get_provider(), self.agent.name
-        )
-        self.streamline_messages.extend(std_msg)
+        self.agent.append_message(message)
 
     def _prepare_files_processing(self, file_command):
         file_paths_str: str = file_command[6:].strip()
@@ -125,6 +123,8 @@ class MessageHandler(Observable):
         if user_input.startswith("/file "):
             self._prepare_files_processing(user_input)
             return False, True
+        if user_input.startswith("/retry"):
+            return False, False
 
         # Process commands first
         command_result = await self.command_processor.process_command(user_input)
@@ -169,7 +169,11 @@ class MessageHandler(Observable):
 
         # Add regular text message
         self._messages_append(
-            {"role": "user", "content": [{"type": "text", "text": user_input}]}
+            {
+                "role": "user",
+                "agent": self.agent.name,
+                "content": [{"type": "text", "text": user_input}],
+            }
         )
         self.current_user_input = self.agent.history[-1]
         self.current_user_input_idx = len(self.streamline_messages) - 1
@@ -208,10 +212,12 @@ class MessageHandler(Observable):
         start_thinking = False
         end_thinking = False
         has_stop_interupted = False
-        voice_sentence = "" if self._is_voice_enabled() else None
-        voice_id = (
-            self.voice_service.get_configured_voice_id() if self.voice_service else None
-        )
+        voice_mode = self._is_voice_enabled()
+        voice_sentence = "" if voice_mode != "disabled" else None
+        voice_id = self._get_configured_voice_id() if self.voice_service else None
+
+        if len(self.agent.history) == 0:
+            return None, 0, 0
 
         # Create a reference to the streaming generator
         self.stream_generator = None
@@ -237,7 +243,7 @@ class MessageHandler(Observable):
                     self.stop_streaming = False  # Reset flag
                     has_stop_interupted = True
                     self._notify("streaming_stopped", assistant_response)
-                    break
+                    await self.stream_generator.aclose()
 
                 # Accumulate thinking content if available
                 if thinking_chunk:
@@ -266,16 +272,44 @@ class MessageHandler(Observable):
                         time.sleep(0.5)
                     self._notify("response_chunk", (chunk_text, assistant_response))
                     if voice_sentence is not None:
-                        voice_sentence += chunk_text
+                        if (
+                            "<agent_evaluation>" in assistant_response
+                            and "</agent_evaluation>" in assistant_response
+                        ) or (
+                            # First token of agent evaluation
+                            "<agent" not in assistant_response
+                            and "</agent_evaluation>" not in assistant_response
+                        ):
+                            voice_sentence += chunk_text
+
                     if (
                         voice_sentence
-                        and "\n" in voice_sentence.strip()
+                        and "\n" in voice_sentence.lstrip("\n ").strip("<>_-")
                         and self.voice_service
+                        and voice_id
                     ):
-                        self.voice_service.text_to_speech_stream(
-                            voice_sentence.strip().partition("\n")[0], voice_id=voice_id
+                        voice_sentence = (
+                            voice_sentence.replace("<agent_evaluation>", "")
+                            .replace("</agent_evaluation>", "")
+                            .replace("<agent", "")
+                            .replace("evaluation>", "")
+                            .lstrip("\n ")
+                            .strip("<>_-")
                         )
-                        voice_sentence = None
+                        print(voice_sentence)
+                        if len(voice_sentence.split("\n")) > 1:
+                            self.voice_service.text_to_speech_stream(
+                                voice_sentence.strip().partition("\n")[0],
+                                voice_id=voice_id,
+                            )
+                            if voice_mode == "partial":
+                                voice_sentence = None
+                            else:
+                                voice_sentence = (
+                                    voice_sentence.strip()
+                                    .partition("\n")[-1]
+                                    .lstrip("\n")
+                                )
 
             # End thinking when break the response stream
             if not end_thinking and start_thinking:
@@ -366,21 +400,20 @@ class MessageHandler(Observable):
             # Final assistant message
             self._notify("response_completed", assistant_response)
 
+            if self.is_non_interactive:
+                # No need to persist or store conversation turn since is single turn task
+                return assistant_response, input_tokens, output_tokens
+
             # --- Start of Persistence Logic ---
             if self.current_conversation_id and self.last_assisstant_response_idx >= 0:
                 try:
-                    # Get all messages added since the user input for this turn
-                    current_provider = self.agent.get_provider()
-                    messages_for_this_turn = MessageTransformer.standardize_messages(
-                        self.agent.history[self.last_assisstant_response_idx :],
-                        current_provider,
-                        self.agent.name,
-                    )
+                    messages_for_this_turn = self.get_recent_agent_responses()
                     if (
                         messages_for_this_turn
                     ):  # Only save if there are messages for the turn
                         self.persistent_service.append_conversation_messages(
-                            self.current_conversation_id, messages_for_this_turn
+                            self.current_conversation_id,
+                            messages_for_this_turn,
                         )
                         self._notify(
                             "conversation_saved", {"id": self.current_conversation_id}
@@ -390,7 +423,7 @@ class MessageHandler(Observable):
                     logger.error(f"ERROR: {error_message}")
                     self._notify("error", {"message": error_message})
 
-            self.last_assisstant_response_idx = len(self.agent.history)
+            self.last_assisstant_response_idx = len(self.streamline_messages)
             # --- End of Persistence Logic ---
 
             if self.current_user_input and self.current_user_input_idx >= 0:
@@ -434,23 +467,18 @@ class MessageHandler(Observable):
                 self.current_user_input = None
                 self.current_user_input_idx = -1
             if self.current_conversation_id and self.last_assisstant_response_idx >= 0:
-                # Get all messages added since the user input for this turn
-                current_provider = self.agent.get_provider()
-                messages_for_this_turn = MessageTransformer.standardize_messages(
-                    self.agent.history[self.last_assisstant_response_idx :],
-                    current_provider,
-                    self.agent.name,
-                )
+                messages_for_this_turn = self.get_recent_agent_responses()
                 if (
                     messages_for_this_turn
                 ):  # Only save if there are messages for the turn
                     self.persistent_service.append_conversation_messages(
-                        self.current_conversation_id, messages_for_this_turn
+                        self.current_conversation_id,
+                        messages_for_this_turn,
                     )
                     self._notify(
                         "conversation_saved", {"id": self.current_conversation_id}
                     )
-            self.last_assisstant_response_idx = len(self.agent.history)
+            self.last_assisstant_response_idx = len(self.streamline_messages)
 
             error_message = str(e)
             traceback_str = traceback.format_exc()
@@ -464,6 +492,9 @@ class MessageHandler(Observable):
             )
             return None, 0, 0
 
+    def get_recent_agent_responses(self) -> List:
+        return self.streamline_messages[self.last_assisstant_response_idx :]
+
     # Delegate conversation management methods
     def list_conversations(self):
         """Lists available conversations from the persistence service."""
@@ -480,11 +511,28 @@ class MessageHandler(Observable):
         return self.conversation_manager.delete_conversation_by_id(conversation_id)
 
     def _is_voice_enabled(self) -> bool:
-        """Check if voice is enabled in global settings."""
+        """Check if voice is enabled in current agent settings."""
         try:
-            config_management = ConfigManagement()
-            global_config = config_management.read_global_config_data()
-            return global_config.get("global_settings", {}).get("voice_enabled", False)
+            # Check if voice service is available first
+            if self.voice_service is None:
+                return False
+
+            if hasattr(self.agent, "voice_enabled"):
+                return getattr(self.agent, "voice_enabled")
+
+            return False
         except Exception as e:
             logger.warning(f"Failed to read voice_enabled setting: {e}")
             return False
+
+    def _get_configured_voice_id(self) -> Optional[str]:
+        """Get the voice ID from current agent settings or return default."""
+        try:
+            if hasattr(self.agent, "voice_id"):
+                return getattr(self.agent, "voice_id", None)
+
+            return None
+
+        except Exception as e:
+            logger.warning(f"Failed to read voice_id from agent config: {e}")
+            return None
