@@ -3,21 +3,17 @@ Main console UI class that orchestrates all console functionality.
 Refactored to use separate modules for different responsibilities.
 """
 
+from __future__ import annotations
 import asyncio
-import sys
 import time
 from typing import Any
 from rich.console import Console
 from rich.text import Text
 
-try:
-    import pyperclip
-except ImportError:
-    pyperclip = None
-
 import AgentCrew
-from AgentCrew.modules.chat.message_handler import MessageHandler, Observer
-from AgentCrew.modules import logger
+from AgentCrew.modules.chat.message_handler import Observer
+from loguru import logger
+from .utils import agent_evaluation_remove
 
 from .constants import (
     RICH_STYLE_GREEN,
@@ -26,12 +22,10 @@ from .constants import (
     RICH_STYLE_YELLOW_BOLD,
 )
 
-from .display_handlers import DisplayHandlers
-from .tool_display import ToolDisplayHandlers
-from .input_handler import InputHandler
-from .ui_effects import UIEffects
-from .confirmation_handler import ConfirmationHandler
-from .conversation_handler import ConversationHandler
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from AgentCrew.modules.chat.message_handler import MessageHandler
 
 
 class ConsoleUI(Observer):
@@ -47,6 +41,15 @@ class ConsoleUI(Observer):
         Args:
             message_handler: The MessageHandler instance that this UI will observe.
         """
+
+        from .display_handlers import DisplayHandlers
+        from .tool_display import ToolDisplayHandlers
+        from .input_handler import InputHandler
+        from .ui_effects import UIEffects
+        from .confirmation_handler import ConfirmationHandler
+        from .conversation_handler import ConversationHandler
+        from .command_handlers import CommandHandlers
+
         self.message_handler = message_handler
         self.voice_recording = False
         self.message_handler.attach(self)
@@ -63,12 +66,11 @@ class ConsoleUI(Observer):
             self.console, self.message_handler, self.display_handlers
         )
         self.ui_effects = UIEffects(self.console, self.message_handler)
-        self.confirmation_handler = ConfirmationHandler(
-            self.console, self.input_handler
-        )
+        self.confirmation_handler = ConfirmationHandler(self, self.input_handler)
         self.conversation_handler = ConversationHandler(
             self.console, self.display_handlers
         )
+        self.command_handlers = CommandHandlers(self.console, self.message_handler)
 
     def listen(self, event: str, data: Any = None):
         """
@@ -89,8 +91,22 @@ class ConsoleUI(Observer):
         elif event == "user_message_created":
             self.console.print("\n")
         elif event == "response_chunk":
-            self.ui_effects.stop_loading_animation()  # Stop loading on first chunk
             _, assistant_response = data
+            if (
+                "<agent_evaluation>" in assistant_response
+                and "</agent_evaluation>" not in assistant_response
+            ):
+                # Skip incomplete evaluation tags
+                return
+            if "<agent_evaluation>" in assistant_response:
+                assistant_response = (
+                    assistant_response[: assistant_response.find("<agent_evaluation>")]
+                    + assistant_response[
+                        assistant_response.find("</agent_evaluation>") + 19 :
+                    ]
+                )
+
+            self.ui_effects.stop_loading_animation()  # Stop loading on first chunk
             self.ui_effects.update_live_display(
                 assistant_response
             )  # data is the response chunk
@@ -114,6 +130,7 @@ class ConsoleUI(Observer):
                 data
             )  # data is the tool use that was denied
         elif event == "response_completed" or event == "assistant_message_added":
+            data = agent_evaluation_remove(data)
             self.ui_effects.finish_response(data)  # data is the complete response
             self.latest_assistant_response = data
         elif event == "error":
@@ -130,7 +147,7 @@ class ConsoleUI(Observer):
             self.display_handlers.display_message(
                 Text("🎮 Ending chat session. Goodbye!", style=RICH_STYLE_YELLOW_BOLD)
             )
-            sys.exit(0)
+            raise SystemExit(0)
         elif event == "copy_requested":
             self.copy_to_clipboard(data)  # data is the text to copy
         elif event == "debug_requested":
@@ -217,6 +234,24 @@ class ConsoleUI(Observer):
             self.display_handlers.display_message(
                 Text("Start recording press Enter to stop...", style="bold yellow")
             )
+        elif event == "voice_activate":
+            if data:
+                asyncio.run(self.message_handler.process_user_input(data))
+                self.input_handler.is_message_processing = True
+                # Get assistant response
+                assistant_response, input_tokens, output_tokens = asyncio.run(
+                    self.message_handler.get_assistant_response()
+                )
+                self.input_handler.is_message_processing = False
+
+                total_cost = self._calculate_token_usage(input_tokens, output_tokens)
+
+                if assistant_response:
+                    # Calculate and display token usage
+                    self.display_token_usage(
+                        input_tokens, output_tokens, total_cost, self.session_cost
+                    )
+
         elif event == "voice_recording_stopping":
             self.display_handlers.display_message(
                 Text("⏹️  Stopping recording...", style="bold yellow")
@@ -229,6 +264,10 @@ class ConsoleUI(Observer):
 
     def copy_to_clipboard(self, text: str):
         """Copy text to clipboard and show confirmation."""
+        try:
+            import pyperclip
+        except ImportError:
+            pyperclip = None
         if text:
             if pyperclip:
                 pyperclip.copy(text)
@@ -288,7 +327,7 @@ class ConsoleUI(Observer):
                 )
             )
             self.input_handler.stop()
-            sys.exit(0)
+            raise SystemExit(0)
         else:
             self._last_ctrl_c_time = current_time
             self.console.print(
@@ -362,7 +401,7 @@ class ConsoleUI(Observer):
                         continue
 
                     # Handle load command directly
-                    if user_input.strip().startswith("/load "):
+                    elif user_input.strip().startswith("/load "):
                         load_arg = user_input.strip()[
                             6:
                         ].strip()  # Extract argument after "/load "
@@ -379,17 +418,85 @@ class ConsoleUI(Observer):
                             )
                         continue
 
-                    # Handle help command directly
-                    if user_input.strip() == "/help":
+                    elif user_input.strip() == "/help":
                         self.console.print("\n")
                         self.print_welcome_message()
                         continue
 
+                    elif user_input.strip().startswith("/export_agent "):
+                        # Extract arguments after "/export_agent "
+                        args = user_input.strip()[14:].strip()
+                        if args:
+                            # Split into agent names and output file
+                            # Expected format: /export_agent <agent1,agent2,...> <output_file>
+                            parts = args.rsplit(maxsplit=1)
+                            if len(parts) == 2:
+                                agent_names, output_file = parts
+                                self.command_handlers.handle_export_agent_command(
+                                    agent_names, output_file
+                                )
+                            else:
+                                self.console.print(
+                                    Text(
+                                        "Usage: /export_agent <agent_names> <output_file>\n"
+                                        "Export selected agents to a TOML file.\n"
+                                        "Agent names should be comma-separated.\n"
+                                        "Example: /export_agent Agent1,Agent2 ./my_agents.toml",
+                                        style=RICH_STYLE_YELLOW,
+                                    )
+                                )
+                        else:
+                            self.console.print(
+                                Text(
+                                    "Usage: /export_agent <agent_names> <output_file>\n"
+                                    "Export selected agents to a TOML file.\n"
+                                    "Agent names should be comma-separated.\n"
+                                    "Example: /export_agent Agent1,Agent2 ./my_agents.toml",
+                                    style=RICH_STYLE_YELLOW,
+                                )
+                            )
+                        continue
+
+                    elif user_input.strip().startswith("/import_agent "):
+                        file_or_url = user_input.strip()[
+                            14:
+                        ].strip()  # Extract argument after "/import_agent "
+                        if file_or_url:
+                            self.command_handlers.handle_import_agent_command(
+                                file_or_url
+                            )
+                        else:
+                            self.console.print(
+                                Text(
+                                    "Usage: /import_agent <file_path_or_url>\nImport/replace agents from file or URL.\nExample: /import_agent ./agents.toml or /import_agent https://example.com/agents.toml",
+                                    style=RICH_STYLE_YELLOW,
+                                )
+                            )
+                        continue
+
+                    # Handle edit_agent command directly
+                    elif user_input.strip() == "/edit_agent":
+                        self.command_handlers.handle_edit_agent_command()
+                        continue
+
+                    # Handle edit_mcp command directly
+                    elif user_input.strip() == "/edit_mcp":
+                        self.command_handlers.handle_edit_mcp_command()
+                        continue
+
+                    # Handle edit_config command directly
+                    elif user_input.strip() == "/edit_config":
+                        self.command_handlers.handle_edit_config_command()
+                        continue
+
                     # Start loading animation while waiting for response
                     if (
                         not user_input.startswith("/")
                         or user_input.startswith("/file ")
                         or user_input.startswith("/consolidate ")
+                        or user_input.startswith("/agent ")
+                        or user_input.startswith("/model ")
+                        or user_input.startswith("/retry ")
                     ):
                         self.start_loading_animation()
 
