@@ -21,7 +21,7 @@ from .constants import (
     RICH_STYLE_GREEN_BOLD,
     RICH_STYLE_BLUE_BOLD,
     RICH_STYLE_FILE_ACCENT_BOLD,
-    RICH_STYLE_FILE_PATH,
+    RICH_STYLE_WHITE,
     CODE_THEME,
 )
 
@@ -241,9 +241,7 @@ class DisplayHandlers:
             return
 
         file_display = Text("📎 Added files: ", style=RICH_STYLE_FILE_ACCENT_BOLD)
-        file_display.append(
-            f"{', '.join(self._added_files)}", style=RICH_STYLE_FILE_PATH
-        )
+        file_display.append(f"{', '.join(self._added_files)}", style=RICH_STYLE_WHITE)
         self.console.print(file_display)
 
     def print_welcome_message(self, version: str):
@@ -288,6 +286,26 @@ class DisplayHandlers:
                 "Use '/agent [agent_name]' to switch agents or list available agents.",
                 style=RICH_STYLE_YELLOW,
             ),
+            Text(
+                "Use '/export_agent <agent_names> <output_file>' to export selected agents to a TOML file (comma-separated names).",
+                style=RICH_STYLE_YELLOW,
+            ),
+            Text(
+                "Use '/import_agent <file_or_url>' to import/replace agent configuration from a file or URL.",
+                style=RICH_STYLE_YELLOW,
+            ),
+            Text(
+                "Use '/edit_agent' to open agent configuration file in your default editor.",
+                style=RICH_STYLE_YELLOW,
+            ),
+            Text(
+                "Use '/edit_mcp' to open MCP configuration file in your default editor.",
+                style=RICH_STYLE_YELLOW,
+            ),
+            Text(
+                "Use '/edit_config' to open AgentCrew global configuration file in your default editor.",
+                style=RICH_STYLE_YELLOW,
+            ),
             Text(
                 "Use '/toggle_transfer' to toggle agent transfer enforcement.",
                 style=RICH_STYLE_YELLOW,
@@ -324,7 +342,11 @@ class DisplayHandlers:
         title.append(":")
         title.append(f"{model_name}]", style=RICH_STYLE_BLUE)
         title.append(
-            "\n(Press Enter for new line, Ctrl+S/Alt+Enter to submit, Up/Down for history)\n",
+            f"            [{datetime.now().strftime('%H:%M:%S')}]",
+            style=RICH_STYLE_GRAY,
+        )
+        title.append(
+            "\n(Press Enter for new line, Ctrl+S/Alt+Enter to Send, Ctrl+V to paste)\n",
             style=RICH_STYLE_YELLOW,
         )
         self.console.print(title)
