@@ -3,7 +3,7 @@ from AgentCrew.modules.openai import OpenAIService
 from AgentCrew.modules.llm.base import AsyncIterator
 from typing import Dict, Any, List, Optional, Tuple
 import json
-from AgentCrew.modules import logger
+from loguru import logger
 
 
 class CustomLLMService(OpenAIService):
@@ -33,97 +33,81 @@ class CustomLLMService(OpenAIService):
         )
         self.extra_headers = extra_headers
 
-    def format_tool_result(
-        self, tool_use: Dict, tool_result: Any, is_error: bool = False
-    ) -> Dict[str, Any]:
-        """
-        Format a tool result for CustomLLMService API.
-
-        Args:
-            tool_use_id: The ID of the tool use
-            tool_result: The result from the tool execution
-            is_error: Whether the result is an error
-
-        Returns:
-            A formatted message that can be appended to the messages list
-        """
-        if isinstance(tool_result, list):
-            parsed_tool_result = []
-            for res in tool_result:
-                # Skipping vision/image tool results for CustomLLMService
-                # if res.get("type", "text") == "image_url":
-                #     if "vision" in ModelRegistry.get_model_capabilities(self.model):
-                #         parsed_tool_result.append(res)
-                # else:
-                if res.get("type", "text") == "text":
-                    parsed_tool_result.append(res.get("text", ""))
-            tool_result = "\n".join(parsed_tool_result) if parsed_tool_result else ""
-        message = {
-            "role": "tool",
-            "tool_call_id": tool_use["id"],
-            "name": tool_use["name"],
-            "content": tool_result,  # Groq and deepinfra expects string content
-        }
-
-        # Add error indication if needed
-        if is_error:
-            message["content"] = f"ERROR: {message['content']}"
-
-        return message
-
     async def process_message(self, prompt: str, temperature: float = 0) -> str:
-        try:
-            response = await self.client.chat.completions.create(
-                model=self.model,
-                timeout=60,
-                max_tokens=3000,
-                temperature=temperature,
-                messages=[
-                    {
-                        "role": "user",
-                        "content": prompt,
-                    }
-                ],
-                extra_headers=self.extra_headers,
-            )
+        response = await self.client.chat.completions.create(
+            model=self.model,
+            timeout=60,
+            max_tokens=3000,
+            temperature=temperature,
+            messages=[
+                {
+                    "role": "user",
+                    "content": prompt,
+                }
+            ],
+            extra_headers=self.extra_headers,
+        )
 
-            # Calculate and log token usage and cost
-            input_tokens = response.usage.prompt_tokens if response.usage else 0
-            output_tokens = response.usage.completion_tokens if response.usage else 0
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate and log token usage and cost
+        input_tokens = response.usage.prompt_tokens if response.usage else 0
+        output_tokens = response.usage.completion_tokens if response.usage else 0
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nToken Usage Statistics:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
-            analyze_result = response.choices[0].message.content or ""
-            if "thinking" in ModelRegistry.get_model_capabilities(
-                f"{self._provider_name}/{self.model}"
+        logger.info("\nToken Usage Statistics:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
+        analyze_result = response.choices[0].message.content or ""
+        if "thinking" in ModelRegistry.get_model_capabilities(
+            f"{self._provider_name}/{self.model}"
+        ):
+            THINK_STARTED = "<think>"
+            THINK_STOPED = "</think>"
+
+            if (
+                analyze_result.find(THINK_STARTED) >= 0
+                and analyze_result.find(THINK_STOPED) >= 0
             ):
-                THINK_STARTED = "<think>"
-                THINK_STOPED = "</think>"
+                analyze_result = (
+                    analyze_result[: analyze_result.find(THINK_STARTED)]
+                    + analyze_result[
+                        (analyze_result.find(THINK_STOPED) + len(THINK_STOPED)) :
+                    ]
+                )
 
-                if (
-                    analyze_result.find(THINK_STARTED) >= 0
-                    and analyze_result.find(THINK_STOPED) >= 0
-                ):
-                    analyze_result = (
-                        analyze_result[: analyze_result.find(THINK_STARTED)]
-                        + analyze_result[
-                            (analyze_result.find(THINK_STOPED) + len(THINK_STOPED)) :
-                        ]
+        return analyze_result
+
+    def _convert_internal_format(self, messages: List[Dict[str, Any]]):
+        for msg in messages:
+            msg.pop("agent", None)
+            if "tool_calls" in msg and msg.get("tool_calls", []):
+                for tool_call in msg["tool_calls"]:
+                    tool_call["function"] = {}
+                    tool_call["function"]["name"] = tool_call.pop("name", "")
+                    tool_call["function"]["arguments"] = json.dumps(
+                        tool_call.pop("arguments", {})
                     )
+            if msg.get("role") == "tool":
+                msg.pop("tool_name", None)
+                if isinstance(msg.get("content", ""), List):
+                    cleaned_tool_content = []
+                    for tool_content in msg["content"]:
+                        if isinstance(tool_content, dict):
+                            if tool_content.get("type", "text") == "text":
+                                cleaned_tool_content.append(
+                                    tool_content.get("text", "")
+                                )
+                    msg["content"] = "\n".join(cleaned_tool_content)
 
-            return analyze_result
-        except Exception as e:
-            raise Exception(f"Failed to process content: {str(e)}")
+        return messages
 
     async def stream_assistant_response(self, messages):
         """Stream the assistant's response with tool support."""
 
         stream_params = {
             "model": self.model,
+            "parallel_tool_calls": False,
             "messages": messages,
             # "max_tokens": 16000,
         }
@@ -132,9 +116,9 @@ class CustomLLMService(OpenAIService):
 
         # Add system message if provided
         if self.system_prompt:
-            stream_params["messages"] = [
-                {"role": "system", "content": self.system_prompt}
-            ] + messages
+            stream_params["messages"] = self._convert_internal_format(
+                [{"role": "system", "content": self.system_prompt}] + messages
+            )
 
         # Add tools if available
         if self.tools and "tool_use" in ModelRegistry.get_model_capabilities(
@@ -148,35 +132,32 @@ class CustomLLMService(OpenAIService):
         ):
             stream_params["reasoning_effort"] = "none"
 
-        try:
-            if "stream" in ModelRegistry.get_model_capabilities(
-                f"{self._provider_name}/{self.model}"
-            ):
-                self._is_thinking = False
-                return await self.client.chat.completions.create(
-                    **stream_params,
-                    stream=True,
-                    extra_headers=self.extra_headers,
-                )
+        if "stream" in ModelRegistry.get_model_capabilities(
+            f"{self._provider_name}/{self.model}"
+        ):
+            self._is_thinking = False
+            return await self.client.chat.completions.create(
+                **stream_params,
+                stream=True,
+                extra_headers=self.extra_headers,
+            )
 
+        else:
+            response = await self.client.chat.completions.create(
+                **stream_params,
+                stream=False,
+                extra_headers=self.extra_headers,
+            )
+
+            if response.usage:
+                self.current_input_tokens = response.usage.prompt_tokens
+                self.current_output_tokens = response.usage.completion_tokens
             else:
-                response = await self.client.chat.completions.create(
-                    **stream_params,
-                    stream=False,
-                    extra_headers=self.extra_headers,
-                )
+                self.current_input_tokens = 0
+                self.current_output_tokens = 0
 
-                if response.usage:
-                    self.current_input_tokens = response.usage.prompt_tokens
-                    self.current_output_tokens = response.usage.completion_tokens
-                else:
-                    self.current_input_tokens = 0
-                    self.current_output_tokens = 0
-
-                # Return an AsyncIterator wrapping response.choices
-                return AsyncIterator(response.choices)
-        except Exception as e:
-            logger.error(f"Error in stream_assistant_response: {str(e)}")
+            # Return an AsyncIterator wrapping response.choices
+            return AsyncIterator(response.choices)
 
     def process_stream_chunk(
         self, chunk, assistant_response: str, tool_uses: List[Dict]
@@ -385,10 +366,8 @@ class CustomLLMService(OpenAIService):
             if delta_tool_calls:
                 # Process each tool call in the delta
                 for tool_call_delta in delta_tool_calls:
-                    tool_call_index = tool_call_delta.index or 0
-
                     # Check if this is a new tool call
-                    if tool_call_index >= len(tool_uses):
+                    if getattr(tool_call_delta, "id"):
                         # Create a new tool call entry
                         tool_uses.append(
                             {
@@ -403,10 +382,11 @@ class CustomLLMService(OpenAIService):
                                 "response": "",
                             }
                         )
+                    tool_call_index = len(tool_uses) - 1
 
-                    # Update existing tool call with new data
-                    if hasattr(tool_call_delta, "id") and tool_call_delta.id:
-                        tool_uses[tool_call_index]["id"] = tool_call_delta.id
+                    # # Update existing tool call with new data
+                    # if hasattr(tool_call_delta, "id") and tool_call_delta.id:
+                    #     tool_uses[tool_call_index]["id"] = tool_call_delta.id
 
                     if hasattr(tool_call_delta, "function"):
                         if (
