@@ -13,7 +13,8 @@ from AgentCrew.modules.llm.base import (
     read_text_file,
     base64_to_bytes,
 )
-from AgentCrew.modules import logger
+from loguru import logger
+import traceback
 
 
 class GoogleStreamAdapter:
@@ -46,12 +47,15 @@ class GoogleStreamAdapter:
     async def __anext__(self):
         """Get the next chunk from the stream generator."""
         try:
-            return await self.stream_generator.__anext__()
+            return self.stream_generator.__next__()
         except StopAsyncIteration:
             raise
+        except StopIteration:
+            raise StopAsyncIteration
         except Exception as e:
             # Handle any Google GenAI specific exceptions
             logger.error(f"Error in Google GenAI stream: {str(e)}")
+            traceback.print_exc()
             raise StopAsyncIteration
 
 
@@ -146,35 +150,32 @@ class GoogleAINativeService(BaseLLMService):
         return 0.0
 
     async def process_message(self, prompt: str, temperature: float = 0) -> str:
-        try:
-            response = await self.client.aio.models.generate_content(
-                model=self.model,
-                contents=prompt,
-                config=types.GenerateContentConfig(
-                    max_output_tokens=3000, temperature=temperature
-                ),
-            )
+        response = await self.client.aio.models.generate_content(
+            model=self.model,
+            contents=prompt,
+            config=types.GenerateContentConfig(
+                max_output_tokens=3000, temperature=temperature
+            ),
+        )
 
-            # Get token usage if available
-            input_tokens = 0
-            output_tokens = 0
-            if hasattr(response, "usage_metadata") and response.usage_metadata:
-                if hasattr(response.usage_metadata, "prompt_token_count"):
-                    input_tokens = response.usage_metadata.prompt_token_count or 0
-                if hasattr(response.usage_metadata, "candidates_token_count"):
-                    output_tokens = response.usage_metadata.candidates_token_count or 0
+        # Get token usage if available
+        input_tokens = 0
+        output_tokens = 0
+        if hasattr(response, "usage_metadata") and response.usage_metadata:
+            if hasattr(response.usage_metadata, "prompt_token_count"):
+                input_tokens = response.usage_metadata.prompt_token_count or 0
+            if hasattr(response.usage_metadata, "candidates_token_count"):
+                output_tokens = response.usage_metadata.candidates_token_count or 0
 
-            # Calculate and log cost
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
-            logger.info("\nToken Usage Statistics:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        # Calculate and log cost
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
+        logger.info("\nToken Usage Statistics:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            return response.text or ""
-        except Exception as e:
-            raise Exception(f"Failed to process content: {str(e)}")
+        return response.text or ""
 
     def process_file_for_message(self, file_path: str) -> Optional[Dict[str, Any]]:
         """
@@ -382,7 +383,7 @@ class GoogleAINativeService(BaseLLMService):
             GoogleStreamAdapter: A context manager compatible adapter
         """
         # Convert messages to Google GenAI format
-        google_messages = self._convert_messages_to_google_format(messages)
+        google_messages = self._convert_internal_format(messages)
 
         # Create configuration with tools
         config = types.GenerateContentConfig(
@@ -407,14 +408,14 @@ class GoogleAINativeService(BaseLLMService):
             )
 
         # Get the stream generator
-        stream_generator = await self.client.aio.models.generate_content_stream(
+        stream_generator = self.client.models.generate_content_stream(
             model=self.model, contents=google_messages, config=config
         )
 
         # Wrap in adapter that supports context manager protocol
         return GoogleStreamAdapter(stream_generator)
 
-    def _convert_messages_to_google_format(self, messages: List[Dict[str, Any]]):
+    def _convert_internal_format(self, messages: List[Dict[str, Any]]):
         """
         Convert standard messages to Google GenAI format.
 
@@ -626,74 +627,6 @@ class GoogleAINativeService(BaseLLMService):
             (thinking_content, None) if thinking_content.strip() else None,
         )
 
-    def format_tool_result(
-        self, tool_use: Dict, tool_result: Any, is_error: bool = False
-    ) -> Dict[str, Any]:
-        """
-        Format a tool result for the Google GenAI API.
-
-        Args:
-            tool_use: The tool use details
-            tool_result: The result from the tool execution
-            is_error: Whether the result is an error
-
-        Returns:
-            A formatted message for tool response
-        """
-        if is_error:
-            tool_result = f"ERROR: {str(tool_result)}"
-
-        # Return in a format expected by the interactive chat
-        return {"role": "tool", "tool_call_id": tool_use["id"], "content": tool_result}
-
-    def format_assistant_message(
-        self, assistant_response: str, tool_uses: Optional[List[Dict]] = None
-    ) -> Dict[str, Any]:
-        """
-        Format the assistant's response for the Google GenAI API.
-
-        Args:
-            assistant_response: The response text
-            tool_uses: List of tool use details
-
-        Returns:
-            Formatted assistant message
-        """
-        if tool_uses and any(tu.get("id") for tu in tool_uses):
-            # Assistant message with tool calls
-            return {
-                "role": "assistant",
-                "content": assistant_response,
-                "tool_calls": [
-                    {
-                        "id": tool_use["id"],
-                        "name": tool_use["name"],
-                        "arguments": tool_use["input"],
-                        "type": tool_use["type"],
-                    }
-                    for tool_use in tool_uses
-                    if tool_use.get("id")  # Only include tool calls with valid IDs
-                ],
-            }
-        else:
-            # Simple assistant message
-            return {"role": "assistant", "content": assistant_response}
-
-    def format_thinking_message(self, thinking_data) -> Optional[Dict[str, Any]]:
-        """
-        Format thinking content into the appropriate message format.
-        Not supported by Google GenAI.
-
-        Args:
-            thinking_data: Tuple containing (thinking_content, thinking_signature)
-                or None if no thinking data is available
-
-        Returns:
-            Dict[str, Any]: A properly formatted message containing thinking blocks
-        """
-        # Google doesn't support thinking blocks
-        return None
-
     async def validate_spec(self, prompt: str) -> str:
         """
         Validate a specification prompt using Google GenAI.
@@ -704,38 +637,33 @@ class GoogleAINativeService(BaseLLMService):
         Returns:
             Validation result as a JSON string
         """
-        try:
-            # Request JSON response
-            response = await self.client.aio.models.generate_content(
-                model=self.model,
-                contents=prompt,
-                config=types.GenerateContentConfig(
-                    response_mime_type="application/json"
-                ),
-            )
+        # Request JSON response
+        response = await self.client.aio.models.generate_content(
+            model=self.model,
+            contents=prompt,
+            config=types.GenerateContentConfig(response_mime_type="application/json"),
+        )
 
-            # Calculate and log token usage
-            input_tokens = 0
-            output_tokens = 0
-            if hasattr(response, "usage_metadata") and response.usage_metadata:
-                if hasattr(response.usage_metadata, "prompt_token_count"):
-                    input_tokens = response.usage_metadata.prompt_token_count or 0
-                if hasattr(response.usage_metadata, "candidates_token_count"):
-                    output_tokens = response.usage_metadata.candidates_token_count or 0
+        # Calculate and log token usage
+        input_tokens = 0
+        output_tokens = 0
+        if hasattr(response, "usage_metadata") and response.usage_metadata:
+            if hasattr(response.usage_metadata, "prompt_token_count"):
+                input_tokens = response.usage_metadata.prompt_token_count or 0
+            if hasattr(response.usage_metadata, "candidates_token_count"):
+                output_tokens = response.usage_metadata.candidates_token_count or 0
 
-            # Calculate cost
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate cost
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nSpec Validation Token Usage:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        logger.info("\nSpec Validation Token Usage:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            # Return the response text (should be JSON)
-            return response.text or ""
-        except Exception as e:
-            raise Exception(f"Failed to validate specification: {str(e)}")
+        # Return the response text (should be JSON)
+        return response.text or ""
 
     def set_system_prompt(self, system_prompt: str):
         """
