@@ -16,7 +16,7 @@ from AgentCrew.modules.llm.base import (
     AsyncIterator,
 )
 from AgentCrew.modules.llm.model_registry import ModelRegistry
-from AgentCrew.modules import logger
+from loguru import logger
 
 
 class GroqService(BaseLLMService):
@@ -65,33 +65,30 @@ class GroqService(BaseLLMService):
         return 0.0
 
     async def process_message(self, prompt: str, temperature: float = 0) -> str:
-        try:
-            response = await self.client.chat.completions.create(
-                model=self.model,
-                max_tokens=3000,
-                temperature=temperature,
-                messages=[
-                    {
-                        "role": "user",
-                        "content": prompt,
-                    }
-                ],
-            )
+        response = await self.client.chat.completions.create(
+            model=self.model,
+            max_tokens=3000,
+            temperature=temperature,
+            messages=[
+                {
+                    "role": "user",
+                    "content": prompt,
+                }
+            ],
+        )
 
-            # Calculate and log token usage and cost
-            input_tokens = response.usage.prompt_tokens if response.usage else 0
-            output_tokens = response.usage.completion_tokens if response.usage else 0
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate and log token usage and cost
+        input_tokens = response.usage.prompt_tokens if response.usage else 0
+        output_tokens = response.usage.completion_tokens if response.usage else 0
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nToken Usage Statistics:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        logger.info("\nToken Usage Statistics:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            return response.choices[0].message.content or ""
-        except Exception as e:
-            raise Exception(f"Failed to process content: {str(e)}")
+        return response.choices[0].message.content or ""
 
     def _process_file(self, file_path):
         mime_type, _ = mimetypes.guess_type(file_path)
@@ -196,6 +193,20 @@ class GroqService(BaseLLMService):
         result = handler(**tool_params)
         return result
 
+    def _convert_internal_format(self, messages: List[Dict[str, Any]]):
+        for msg in messages:
+            msg.pop("agent", None)
+            msg.pop("tool_name", None)
+            if "tool_calls" in msg and msg.get("tool_calls", []):
+                for tool_call in msg["tool_calls"]:
+                    tool_call["function"] = {}
+                    tool_call["function"]["name"] = tool_call.pop("name", "")
+                    tool_call["function"]["arguments"] = json.dumps(
+                        tool_call.pop("arguments", {})
+                    )
+
+        return messages
+
     async def stream_assistant_response(self, messages):
         """Stream the assistant's response with tool support."""
         stream_params = {
@@ -209,13 +220,16 @@ class GroqService(BaseLLMService):
         # Add system message if provided
         if self.system_prompt:
             system_role = "user" if "deepseek" in self.model else "system"
-            stream_params["messages"] = [
-                {
-                    "role": f"{system_role}",
-                    "content": """DO NOT generate Chinese characters.""",
-                },
-                {"role": f"{system_role}", "content": self.system_prompt},
-            ] + messages
+            stream_params["messages"] = self._convert_internal_format(
+                [
+                    {
+                        "role": f"{system_role}",
+                        "content": """DO NOT generate Chinese characters.""",
+                    },
+                    {"role": f"{system_role}", "content": self.system_prompt},
+                ]
+                + messages
+            )
 
         if "thinking" in ModelRegistry.get_model_capabilities(
             f"{self._provider_name}/{self.model}"
@@ -452,86 +466,6 @@ class GroqService(BaseLLMService):
 
         return content, tool_uses
 
-    def format_tool_result(
-        self, tool_use: Dict, tool_result: Any, is_error: bool = False
-    ) -> Dict[str, Any]:
-        """
-        Format a tool result for Groq API.
-
-        Args:
-            tool_use_id: The ID of the tool use
-            tool_result: The result from the tool execution
-            is_error: Whether the result is an error
-
-        Returns:
-            A formatted message that can be appended to the messages list
-        """
-        # Groq follows OpenAI format for tool responses
-        if isinstance(tool_result, list):
-            parsed_tool_result = []
-            for res in tool_result:
-                # Skipping vision/image tool results for Groq
-                # if res.get("type", "text") == "image_url":
-                #     if "vision" in ModelRegistry.get_model_capabilities(self.model):
-                #         parsed_tool_result.append(res)
-                # else:
-                if res.get("type", "text") == "text":
-                    parsed_tool_result.append(res.get("text", ""))
-            tool_result = "\n".join(parsed_tool_result) if parsed_tool_result else ""
-        message = {
-            "role": "tool",
-            "tool_call_id": tool_use["id"],
-            "name": tool_use["name"],
-            "content": tool_result,  # Groq and deepinfra expects string content
-        }
-
-        # Add error indication if needed
-        if is_error:
-            message["content"] = f"ERROR: {message['content']}"
-
-        return message
-
-    def format_assistant_message(
-        self, assistant_response: str, tool_uses: list[Dict] | None = None
-    ) -> Dict[str, Any]:
-        """Format the assistant's response for Groq API."""
-        # Groq uses a simpler format with just a string content
-        if tool_uses:
-            return {
-                "role": "assistant",
-                "content": assistant_response,
-                "tool_calls": [
-                    {
-                        "id": tool_use["id"],
-                        "function": {
-                            "name": tool_use["name"],
-                            "arguments": json.dumps(tool_use["input"]),
-                        },
-                        "type": tool_use["type"],
-                    }
-                    for tool_use in tool_uses
-                ],
-            }
-        else:
-            return {
-                "role": "assistant",
-                "content": assistant_response,
-            }
-
-    def format_thinking_message(self, thinking_data) -> Optional[Dict[str, Any]]:
-        """
-        Format thinking content into the appropriate message format for Groq.
-
-        Args:
-            thinking_data: Tuple containing (thinking_content, thinking_signature)
-                or None if no thinking data is available
-
-        Returns:
-            Dict[str, Any]: A properly formatted message containing thinking blocks
-        """
-        # Groq doesn't support thinking blocks, so we return None
-        return None
-
     async def validate_spec(self, prompt: str) -> str:
         """
         Validate a specification prompt using Groq.
@@ -543,53 +477,47 @@ class GroqService(BaseLLMService):
             Validation result as a JSON string
         """
 
-        try:
-            response = await self.client.chat.completions.create(
-                model=self.model,
-                max_completion_tokens=8192,
-                temperature=0.6,
-                top_p=0.95,
-                reasoning_format="parsed",
-                messages=[
-                    {
-                        "role": "user",
-                        "content": prompt,
-                    },
-                ],
-                # Groq doesn't support response_format, so we rely on the prompt
-            )
+        response = await self.client.chat.completions.create(
+            model=self.model,
+            max_completion_tokens=8192,
+            temperature=0.6,
+            top_p=0.95,
+            reasoning_format="parsed",
+            messages=[
+                {
+                    "role": "user",
+                    "content": prompt,
+                },
+            ],
+            # Groq doesn't support response_format, so we rely on the prompt
+        )
 
-            # Calculate and log token usage and cost
-            input_tokens = response.usage.prompt_tokens if response.usage else 0
-            output_tokens = response.usage.completion_tokens if response.usage else 0
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate and log token usage and cost
+        input_tokens = response.usage.prompt_tokens if response.usage else 0
+        output_tokens = response.usage.completion_tokens if response.usage else 0
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nSpec Validation Token Usage:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        logger.info("\nSpec Validation Token Usage:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            text = response.choices[0].message.content
-            if text is None:
-                raise ValueError("Cannot validate this spec")
-            think_tag = "<think>"
-            end_think_tag = "</think>"
-            think_start_idx = text.find(think_tag)
-            think_end_idx = text.rfind(end_think_tag)
-            if think_start_idx > -1 and think_end_idx > -1:
-                text = (
-                    text[:think_start_idx] + text[think_end_idx + len(end_think_tag) :]
-                )
-            start_tag = "<SpecificationReview>"
-            end_tag = "</SpecificationReview>"
-            start_idx = text.rindex(start_tag)
-            end_idx = text.rindex(end_tag) + len(end_tag)
-            result = text[start_idx:end_idx].strip()
-            return result
-
-        except Exception as e:
-            raise Exception(f"Failed to validate specification: {str(e)}")
+        text = response.choices[0].message.content
+        if text is None:
+            raise ValueError("Cannot validate this spec")
+        think_tag = "<think>"
+        end_think_tag = "</think>"
+        think_start_idx = text.find(think_tag)
+        think_end_idx = text.rfind(end_think_tag)
+        if think_start_idx > -1 and think_end_idx > -1:
+            text = text[:think_start_idx] + text[think_end_idx + len(end_think_tag) :]
+        start_tag = "<SpecificationReview>"
+        end_tag = "</SpecificationReview>"
+        start_idx = text.rindex(start_tag)
+        end_idx = text.rindex(end_tag) + len(end_tag)
+        result = text[start_idx:end_idx].strip()
+        return result
 
     def set_system_prompt(self, system_prompt: str):
         """
