@@ -24,14 +24,13 @@ from PySide6.QtCore import (
 )
 from PySide6.QtGui import QIcon
 from AgentCrew.modules.chat.message_handler import MessageHandler, Observer
-from AgentCrew.modules.gui.widgets import ConversationSidebar, TokenUsageWidget
-from AgentCrew.modules.gui.widgets import MessageBubble
-import asyncio
-from AgentCrew.modules import logger
+from .widgets import ConversationSidebar, TokenUsageWidget
+from .widgets import MessageBubble
+from loguru import logger
 
 
-from AgentCrew.modules.gui.worker import LLMWorker
-from AgentCrew.modules.gui.components import (
+from .worker import LLMWorker
+from .components import (
     MenuBuilder,
     KeyboardHandler,
     MessageEventHandler,
@@ -42,7 +41,7 @@ from AgentCrew.modules.gui.components import (
     ConversationComponents,
     CommandHandler,
 )
-from AgentCrew.modules.gui.themes import StyleProvider
+from .themes import StyleProvider
 
 
 class ChatWindow(QMainWindow, Observer):
@@ -257,7 +256,8 @@ class ChatWindow(QMainWindow, Observer):
             return  # Command was processed locally
 
         # Add user message to chat
-        self._add_user_message_bubble(user_input)
+        if user_input.strip() != "/retry":
+            self._add_user_message_bubble(user_input)
 
         # Update status bar
         self.display_status_message("Processing your message...")
@@ -333,24 +333,23 @@ class ChatWindow(QMainWindow, Observer):
 
     def stop_message_stream(self):
         """Stop the current message stream."""
+        if self.message_handler.voice_service:
+            self.message_handler.voice_service.clear_tts_queue()
+            self.input_components.stop_voice_recording()
         if self.waiting_for_response:
             self.ui_state_manager.stop_button_stopping_state()
             if self.message_handler.stream_generator:
                 try:
-                    asyncio.run(self.message_handler.stream_generator.aclose())
+                    self.message_handler.stop_streaming = True
                 except RuntimeError as e:
                     logger.warning(f"Error closing stream generator: {e}")
                 except Exception as e:
                     logger.warning(f"Exception closing stream generator: {e}")
-                finally:
-                    self.message_handler.stop_streaming = True
-                    self.message_handler.stream_generator = None
-            # Also stop UI streaming
-            self.ui_state_manager.set_input_controls_enabled(True)
-            if self.current_response_bubble:
-                self.current_response_bubble.stop_streaming()
-            if self.current_thinking_bubble:
-                self.current_thinking_bubble.stop_streaming()
+        self.ui_state_manager.set_input_controls_enabled(True)
+        if self.current_response_bubble:
+            self.current_response_bubble.stop_streaming()
+        if self.current_thinking_bubble:
+            self.current_thinking_bubble.stop_streaming()
             self.display_status_message("Stopping message stream...")
 
     def show_context_menu(self, position):
@@ -436,7 +435,6 @@ class ChatWindow(QMainWindow, Observer):
             )
             return
 
-        print("Test")
         # Execute the unconsolidate command
         self.llm_worker.process_request.emit("/unconsolidate")
 
@@ -497,6 +495,7 @@ class ChatWindow(QMainWindow, Observer):
             "copy_requested",
             "debug_requested",
             "agent_changed",
+            "agent_command_result",
             "model_changed",
             "think_budget_set",
             "jump_performed",
@@ -560,21 +559,33 @@ class ChatWindow(QMainWindow, Observer):
             self._update_cost_info(data["input_tokens"], data["output_tokens"])
         elif event == "mcp_prompt":
             self.message_input.setPlainText(data.get("content", ""))
+        elif event == "transfer_enforce_toggled":
+            self.chat_components.add_system_message(
+                f"🔄 Transfer enforcement is now {data}."
+            )
         elif event == "voice_recording_started":
             # Update UI to show recording state
             self.ui_state_manager.set_input_controls_enabled(False)
+            self.ui_state_manager.is_voice_activated = True
             self.message_input.setPlaceholderText(
                 "🎤 Recording... Click voice button or press Enter to stop"
             )
             # Update voice button to show recording state
             self.input_components.update_voice_button_state(True)
-        elif event == "voice_recording_completed":
-            # Restore normal UI state
-            self.message_input.setPlaceholderText("Type a message...")
-            # Update voice button to show normal state
-            self.input_components.update_voice_button_state(False)
+        elif event == "voice_activate":
+            if data:
+                self._add_user_message_bubble(data)
+            self.llm_worker.process_request.emit(data)
             self.ui_state_manager._set_send_button_state(True)
-            self._add_user_message_bubble(data)
+        # elif event == "voice_recording_completed":
+        #     self.ui_state_manager.is_voice_activated = False
+        #     # Restore normal UI state
+        #     self.message_input.setPlaceholderText("Type a message...")
+        #     # Update voice button to show normal state
+        #     self.input_components.update_voice_button_state(False)
+        #     self.ui_state_manager.set_input_controls_enabled(
+        #         self.ui_state_manager._last_enabled_state
+        #     )
 
     def _add_user_message_bubble(self, data):
         self.chat_components.append_message(
