@@ -19,8 +19,6 @@ from PySide6.QtWidgets import (
     QTabWidget,
 )
 import os
-import toml
-import json
 from PySide6.QtCore import Qt, Signal
 from PySide6.QtGui import QDoubleValidator
 
@@ -30,6 +28,8 @@ from AgentCrew.modules.memory.context_persistent import ContextPersistenceServic
 
 from AgentCrew.modules.gui.themes import StyleProvider
 from AgentCrew.modules.gui.widgets.markdown_editor import MarkdownEditor
+from AgentCrew.modules.gui.widgets.loading_overlay import LoadingOverlay
+from .save_worker import SaveWorker
 
 
 class AgentsConfigTab(QWidget):
@@ -49,6 +49,9 @@ class AgentsConfigTab(QWidget):
             "code_analysis",
             "web_search",
             "image_generation",
+            "browser",
+            "file_editing",
+            "command_execution",
         ]
 
         # Load agents configuration
@@ -56,6 +59,8 @@ class AgentsConfigTab(QWidget):
         self._is_dirty = False
         self.current_agent_behaviors = {}  # Cache for current agent's behaviors
 
+        self.save_worker = None
+
         self.init_ui()
         self.load_agents()
 
@@ -183,6 +188,30 @@ class AgentsConfigTab(QWidget):
         self.enabled_checkbox.setChecked(True)  # Default to enabled
         local_form_layout.addRow("", self.enabled_checkbox)
 
+        # Voice Settings
+        voice_group = QGroupBox("Voice Settings")
+        voice_layout = QFormLayout()
+
+        self.voice_enabled_checkbox = QCheckBox("Voice Enabled")
+        self.voice_enabled_checkbox.setTristate(True)
+        # Apply enhanced styling for tristate checkbox
+        style_provider = StyleProvider()
+        self.voice_enabled_checkbox.setStyleSheet(style_provider.get_checkbox_style())
+        # Add tooltip explaining the three states
+        self.voice_enabled_checkbox.setToolTip(
+            "Voice Features:\n"
+            "• Square (green): Full voice mode - TTS for full conversation\n"
+            "• Circle (amber): Partial voice mode - TTS for first sentence only\n"
+            "• Unchecked: Voice disabled"
+        )
+        voice_layout.addRow("", self.voice_enabled_checkbox)
+
+        self.voice_id_input = QLineEdit()
+        self.voice_id_input.setPlaceholderText("e.g., kHhWB9Fw3aF6ly7JvltC")
+        voice_layout.addRow("Voice ID:", self.voice_id_input)
+
+        voice_group.setLayout(voice_layout)
+
         tools_group = QGroupBox("Tools")
         tools_layout = QVBoxLayout()
         self.tool_checkboxes = {}
@@ -193,6 +222,7 @@ class AgentsConfigTab(QWidget):
         tools_group.setLayout(tools_layout)
 
         general_tab_layout.addLayout(local_form_layout)
+        general_tab_layout.addWidget(voice_group)
         general_tab_layout.addWidget(tools_group)
         general_tab_layout.addStretch()
 
@@ -342,6 +372,9 @@ class AgentsConfigTab(QWidget):
         self.editor_layout.addWidget(self.save_btn)
         # self.editor_layout.addStretch() # Removed, stretch is within individual editors
 
+        # Create loading overlay (will be parented to the main widget)
+        self.loading_overlay = LoadingOverlay(self, "Saving agents...")
+
         # Connect signals for editor fields to handle changes
         # Local agent fields
         self.name_input.textChanged.connect(self._on_editor_field_changed)
@@ -349,6 +382,11 @@ class AgentsConfigTab(QWidget):
         self.temperature_input.textChanged.connect(self._on_editor_field_changed)
         self.system_prompt_input.markdown_changed.connect(self._on_editor_field_changed)
         self.enabled_checkbox.stateChanged.connect(self._on_editor_field_changed)
+
+        # Voice settings field connections
+        self.voice_enabled_checkbox.stateChanged.connect(self._on_editor_field_changed)
+        self.voice_id_input.textChanged.connect(self._on_editor_field_changed)
+
         for checkbox in self.tool_checkboxes.values():
             checkbox.stateChanged.connect(self._on_editor_field_changed)
         # Behavior editing fields
@@ -421,6 +459,8 @@ class AgentsConfigTab(QWidget):
             self.temperature_input,
             self.system_prompt_input,
             self.enabled_checkbox,
+            self.voice_enabled_checkbox,
+            self.voice_id_input,
             self.remote_name_input,
             self.remote_base_url_input,
             self.remote_enabled_checkbox,
@@ -434,6 +474,20 @@ class AgentsConfigTab(QWidget):
             self.description_input.setText(agent_data.get("description", ""))
             self.temperature_input.setText(str(agent_data.get("temperature", "0.5")))
             self.enabled_checkbox.setChecked(agent_data.get("enabled", True))
+
+            # Load voice settings
+            voice_state = agent_data.get("voice_enabled", "disabled")
+            if voice_state == "full":
+                self.voice_enabled_checkbox.setCheckState(Qt.CheckState.Checked)
+            elif voice_state == "partial":
+                self.voice_enabled_checkbox.setCheckState(
+                    Qt.CheckState.PartiallyChecked
+                )
+            else:
+                self.voice_enabled_checkbox.setCheckState(Qt.CheckState.Unchecked)
+
+            self.voice_id_input.setText(agent_data.get("voice_id", ""))
+
             tools = agent_data.get("tools", [])
             for tool, checkbox in self.tool_checkboxes.items():
                 checkbox.setChecked(tool in tools)
@@ -461,6 +515,11 @@ class AgentsConfigTab(QWidget):
             self.temperature_input.clear()
             self.system_prompt_input.clear()
             self.enabled_checkbox.setChecked(True)  # Default for clearing
+
+            # Clear voice settings
+            self.voice_enabled_checkbox.setChecked(False)
+            self.voice_id_input.clear()
+
             for checkbox in self.tool_checkboxes.values():
                 checkbox.setChecked(False)
             # Clear behaviors
@@ -741,6 +800,16 @@ class AgentsConfigTab(QWidget):
                 return
 
             tools = [t for t, cb in self.tool_checkboxes.items() if cb.isChecked()]
+
+            voice_state = "disabled"
+            if self.voice_enabled_checkbox.checkState() == Qt.CheckState.Checked:
+                voice_state = "full"
+            elif (
+                self.voice_enabled_checkbox.checkState()
+                == Qt.CheckState.PartiallyChecked
+            ):
+                voice_state = "partial"
+
             updated_agent_data = {
                 "name": name,
                 "description": description,
@@ -748,6 +817,8 @@ class AgentsConfigTab(QWidget):
                 "tools": tools,
                 "system_prompt": system_prompt,
                 "enabled": self.enabled_checkbox.isChecked(),
+                "voice_enabled": voice_state,
+                "voice_id": self.voice_id_input.text().strip(),
                 "agent_type": "local",
             }
             current_item.setText(name)
@@ -814,8 +885,8 @@ class AgentsConfigTab(QWidget):
             )
             return
 
-        # Load the configuration file
         try:
+            # First, try to load to check for conflicts
             temp_config = ConfigManagement(import_file_path)
             imported_config = temp_config.get_config()
 
@@ -831,150 +902,100 @@ class AgentsConfigTab(QWidget):
                 )
                 return
 
+            # Check for conflicts
+            existing_agent_names = set()
+            for i in range(self.agents_list.count()):
+                item = self.agents_list.item(i)
+                agent_data = item.data(Qt.ItemDataRole.UserRole)
+                existing_agent_names.add(agent_data.get("name", ""))
+
+            # Find conflicts
+            conflict_names = []
+            imported_names = []
+
+            for agent in local_agents:
+                name = agent.get("name", "")
+                if name:
+                    imported_names.append(name)
+                    if name in existing_agent_names:
+                        conflict_names.append(name)
+
+            for agent in remote_agents:
+                name = agent.get("name", "")
+                if name:
+                    imported_names.append(name)
+                    if name in existing_agent_names:
+                        conflict_names.append(name)
+
+            # If there are conflicts, ask user how to proceed
+            skip_conflicts = False
+            if conflict_names:
+                conflict_list = "\n".join([f"• {name}" for name in conflict_names])
+
+                message_box = QMessageBox(self)
+                message_box.setWindowTitle("Agent Name Conflicts")
+                message_box.setIcon(QMessageBox.Icon.Warning)
+                message_box.setText(
+                    f"The following agent(s) already exist and will be overridden:\n\n"
+                    f"{conflict_list}\n\n"
+                    f"How would you like to proceed?"
+                )
+
+                override_btn = message_box.addButton(
+                    "Override", QMessageBox.ButtonRole.AcceptRole
+                )
+                skip_btn = message_box.addButton(
+                    "Skip Conflicts", QMessageBox.ButtonRole.ActionRole
+                )
+
+                message_box.exec()
+
+                clicked_button = message_box.clickedButton()
+                if clicked_button == override_btn:
+                    skip_conflicts = False
+                elif clicked_button == skip_btn:
+                    skip_conflicts = True
+                else:
+                    return  # User canceled
+
+            # Use ConfigManagement to import agents
+            result = self.config_manager.import_agents(
+                import_file_path, merge_strategy="update", skip_conflicts=skip_conflicts
+            )
+
+            if not result["success"]:
+                QMessageBox.critical(
+                    self,
+                    "Import Error",
+                    f"Failed to import agents:\n{result.get('error', 'Unknown error')}",
+                )
+                return
+
+            # Reload agents in UI
+            self.load_agents()
+
+            # Select the first imported agent in the list if any were imported
+            if result["imported_agents"]:
+                index = self._find_agent_index_by_name(result["imported_agents"][0])
+                if index >= 0:
+                    self.agents_list.setCurrentRow(index)
+
+            # Show success message
+            status_message = f"Successfully imported {result['added_count'] + result['updated_count']} agent(s)."
+
+            if result["added_count"] > 0:
+                status_message += f"\n• Added: {result['added_count']}"
+            if result["updated_count"] > 0:
+                status_message += f"\n• Updated: {result['updated_count']}"
+            if result["skipped_count"] > 0:
+                status_message += f"\n• Skipped: {result['skipped_count']}"
+
+            QMessageBox.information(self, "Import Complete", status_message)
+
         except Exception as e:
             QMessageBox.critical(
-                self, "Import Error", f"Failed to load agent configuration: {str(e)}"
+                self, "Import Error", f"Failed to import agent configuration: {str(e)}"
             )
-            return
-
-        # Check for conflicts
-        existing_agent_names = set()
-        for i in range(self.agents_list.count()):
-            item = self.agents_list.item(i)
-            agent_data = item.data(Qt.ItemDataRole.UserRole)
-            existing_agent_names.add(agent_data.get("name", ""))
-
-        # Find conflicts
-        conflict_names = []
-        imported_names = []
-
-        for agent in local_agents:
-            name = agent.get("name", "")
-            if name:
-                imported_names.append(name)
-                if name in existing_agent_names:
-                    conflict_names.append(name)
-
-        for agent in remote_agents:
-            name = agent.get("name", "")
-            if name:
-                imported_names.append(name)
-                if name in existing_agent_names:
-                    conflict_names.append(name)
-
-        # If there are conflicts, show warning dialog
-        user_choice = "import_all"  # Default: import all
-        if conflict_names:
-            conflict_list = "\n".join([f"• {name}" for name in conflict_names])
-
-            message_box = QMessageBox(self)
-            message_box.setWindowTitle("Agent Name Conflicts")
-            message_box.setIcon(QMessageBox.Icon.Warning)
-            message_box.setText(
-                f"The following agent(s) already exist and will be overridden:\n\n"
-                f"{conflict_list}\n\n"
-                f"How would you like to proceed?"
-            )
-
-            override_btn = message_box.addButton(
-                "Override", QMessageBox.ButtonRole.AcceptRole
-            )
-            skip_btn = message_box.addButton(
-                "Skip Conflicts", QMessageBox.ButtonRole.ActionRole
-            )
-            _ = message_box.addButton("Cancel", QMessageBox.ButtonRole.RejectRole)
-
-            message_box.exec()
-
-            clicked_button = message_box.clickedButton()
-            if clicked_button == override_btn:
-                user_choice = "import_all"
-            elif clicked_button == skip_btn:
-                user_choice = "skip_conflicts"
-            else:
-                user_choice = "cancel"
-
-        if user_choice == "cancel":
-            return
-
-        # Process the import based on user's choice
-        imported_count = 0
-        skipped_count = 0
-
-        # Get current config to update
-        current_local_agents = self.agents_config.get("agents", [])
-        current_remote_agents = self.agents_config.get("remote_agents", [])
-
-        # Process local agents
-        for imported_agent in local_agents:
-            name = imported_agent.get("name", "")
-            if not name:
-                continue
-
-            is_conflict = name in existing_agent_names
-
-            if is_conflict and user_choice == "skip_conflicts":
-                skipped_count += 1
-                continue
-
-            if is_conflict:
-                current_local_agents = [
-                    a for a in current_local_agents if a.get("name") != name
-                ]
-                current_remote_agents = [
-                    a for a in current_remote_agents if a.get("name") != name
-                ]
-
-            if "enabled" not in imported_agent:
-                imported_agent["enabled"] = True
-
-            current_local_agents.append(imported_agent)
-            imported_count += 1
-
-        for imported_agent in remote_agents:
-            name = imported_agent.get("name", "")
-            if not name:
-                continue
-
-            is_conflict = name in existing_agent_names
-
-            if is_conflict and user_choice == "skip_conflicts":
-                skipped_count += 1
-                continue
-
-            if is_conflict:
-                current_local_agents = [
-                    a for a in current_local_agents if a.get("name") != name
-                ]
-                current_remote_agents = [
-                    a for a in current_remote_agents if a.get("name") != name
-                ]
-
-            if "enabled" not in imported_agent:
-                imported_agent["enabled"] = True
-
-            current_remote_agents.append(imported_agent)
-            imported_count += 1
-
-        # Update the configuration
-        self.agents_config["agents"] = current_local_agents
-        self.agents_config["remote_agents"] = current_remote_agents
-
-        # Save the updated configuration and refresh the UI
-        self.config_manager.write_agents_config(self.agents_config)
-        self.load_agents()
-
-        # Select the first imported agent in the list if any were imported
-        if imported_count > 0 and imported_names:
-            index = self._find_agent_index_by_name(imported_names[0])
-            if index >= 0:
-                self.agents_list.setCurrentRow(index)
-
-        status_message = f"Successfully imported {imported_count} agent(s)."
-        if skipped_count > 0:
-            status_message += f" Skipped {skipped_count} agent(s) due to conflicts."
-
-        QMessageBox.information(self, "Import Complete", status_message)
 
     def export_agents(self):
         """Export selected agents to a file."""
@@ -985,29 +1006,27 @@ class AgentsConfigTab(QWidget):
             )
             return
 
-        selected_agents_data = []
-        selected_remote_agents_data = []
-
+        # Collect agent names
+        agent_names = []
         for item in selected_items:
             agent_data = item.data(Qt.ItemDataRole.UserRole)
-            agent_type = agent_data.get("agent_type", "local")
+            agent_name = agent_data.get("name")
+            if agent_name:
+                agent_names.append(agent_name)
 
-            export_data = agent_data.copy()
-            export_data.pop("agent_type", None)
-
-            if agent_type == "local":
-                selected_agents_data.append(export_data)
-            elif agent_type == "remote":
-                selected_remote_agents_data.append(export_data)
-
-        if len(selected_items) == 1:
-            agent_name = (
-                selected_items[0].data(Qt.ItemDataRole.UserRole).get("name", "agent")
+        if not agent_names:
+            QMessageBox.warning(
+                self, "No Agents", "No valid agents selected for export."
             )
-            default_filename = f"{agent_name}_export"
+            return
+
+        # Determine default filename
+        if len(selected_items) == 1:
+            default_filename = f"{agent_names[0]}_export"
         else:
             default_filename = f"agents_export_{len(selected_items)}_agents"
 
+        # File dialog
         file_dialog = QFileDialog(self)
         file_dialog.setWindowTitle("Export Agent Configuration")
         file_dialog.setFileMode(QFileDialog.FileMode.AnyFile)
@@ -1030,33 +1049,51 @@ class AgentsConfigTab(QWidget):
         )
 
         try:
-            export_config = {}
-            if selected_agents_data:
-                export_config["agents"] = selected_agents_data
-            if selected_remote_agents_data:
-                export_config["remote_agents"] = selected_remote_agents_data
-
-            with open(export_file_path, "w", encoding="utf-8") as f:
-                if file_format == "toml":
-                    toml.dump(export_config, f)
-                else:
-                    json.dump(export_config, f, indent=2, ensure_ascii=False)
-
-            agent_count = len(selected_items)
-            agent_word = "agent" if agent_count == 1 else "agents"
-            QMessageBox.information(
-                self,
-                "Export Successful",
-                f"Successfully exported {agent_count} {agent_word} to:\n{export_file_path}",
+            # Use ConfigManagement to export agents
+            result = self.config_manager.export_agents(
+                agent_names, export_file_path, file_format=file_format
             )
 
+            if not result["success"]:
+                QMessageBox.critical(
+                    self,
+                    "Export Error",
+                    f"Failed to export agents:\n{result.get('error', 'Unknown error')}",
+                )
+                return
+
+            # Show success message
+            agent_count = result["exported_count"]
+            agent_word = "agent" if agent_count == 1 else "agents"
+
+            message = f"Successfully exported {agent_count} {agent_word} to:\n{result['output_file']}"
+
+            if result["missing_agents"]:
+                message += (
+                    "\n\nWarning: The following agents were not found:\n"
+                    + "\n".join(result["missing_agents"])
+                )
+
+            QMessageBox.information(self, "Export Successful", message)
+
         except Exception as e:
             QMessageBox.critical(
                 self, "Export Error", f"Failed to export agents:\n{str(e)}"
             )
 
     def save_all_agents(self):
-        """Save all agents to the configuration file."""
+        """Save all agents to the configuration file with loading indicator."""
+        self.loading_overlay.set_message("Saving agents...")
+        self.loading_overlay.show_loading()
+
+        # Disable UI during save
+        self.agents_list.setEnabled(False)
+        self.save_btn.setEnabled(False)
+        self.add_agent_menu_btn.setEnabled(False)
+        self.import_agents_btn.setEnabled(False)
+        self.export_agents_btn.setEnabled(False)
+        self.remove_agent_btn.setEnabled(False)
+
         local_agents_list = []
         remote_agents_list = []
 
@@ -1075,9 +1112,49 @@ class AgentsConfigTab(QWidget):
         self.agents_config["agents"] = local_agents_list
         self.agents_config["remote_agents"] = remote_agents_list
 
-        self.config_manager.write_agents_config(self.agents_config)
+        self.save_worker = SaveWorker(self._perform_agent_save, self.agents_config)
+        self.save_worker.finished.connect(self._on_save_complete)
+        self.save_worker.error.connect(self._on_save_error)
+        self.save_worker.start()
+
+    def _perform_agent_save(self, config_data):
+        """Perform the actual save operation (runs in worker thread)."""
+        self.config_manager.write_agents_config(config_data)
+
+    def _on_save_complete(self):
+        """Handle successful save completion."""
+        self.loading_overlay.hide_loading()
+
+        self.agents_list.setEnabled(True)
+        self.add_agent_menu_btn.setEnabled(True)
+        self.import_agents_btn.setEnabled(True)
+
+        self.on_selection_changed()
+
         self.config_changed.emit()
 
+        if self.save_worker:
+            self.save_worker.deleteLater()
+            self.save_worker = None
+
+    def _on_save_error(self, error_message: str):
+        """Handle save error."""
+        # Hide loading overlay
+        self.loading_overlay.hide_loading()
+
+        self.agents_list.setEnabled(True)
+        self.add_agent_menu_btn.setEnabled(True)
+        self.import_agents_btn.setEnabled(True)
+        self.on_selection_changed()
+
+        QMessageBox.critical(
+            self, "Save Error", f"Failed to save agents configuration:\n{error_message}"
+        )
+
+        if self.save_worker:
+            self.save_worker.deleteLater()
+            self.save_worker = None
+
     def load_agent_behaviors(self, agent_name: str):
         """Load adaptive behaviors for the selected agent."""
         if not agent_name:
@@ -1162,11 +1239,11 @@ class AgentsConfigTab(QWidget):
 
         # Validate behavior format
         behavior_lower = behavior_text.lower()
-        if not behavior_lower.startswith("when ") or " do " not in behavior_lower:
+        if not behavior_lower.startswith("when "):
             QMessageBox.warning(
                 self,
                 "Format Error",
-                "Behavior must follow 'when [condition] do [action]' format.",
+                "Behavior must follow 'when [condition], [action]' format.",
             )
             return
 
