@@ -21,6 +21,16 @@ _ANTHROPIC_MODELS = [
         input_token_price_1m=3.0,
         output_token_price_1m=15.0,
     ),
+    Model(
+        id="claude-sonnet-4-5",
+        provider="claude",
+        name="Claude 4.5 Sonnet",
+        description="Anthropic's best coding model in the world",
+        capabilities=["thinking", "tool_use", "vision", "stream"],
+        default=False,
+        input_token_price_1m=3.0,
+        output_token_price_1m=15.0,
+    ),
     Model(
         id="claude-3-7-sonnet-latest",
         provider="claude",
@@ -62,6 +72,16 @@ _OPENAI_MODELS = [
         output_token_price_1m=10,
         default=True,
     ),
+    Model(
+        id="gpt-5-codex",
+        provider="openai",
+        name="GPT-5 Codex",
+        description="GPT-5-Codex is a version of GPT-5 optimized for agentic coding tasks in Codex or similar environments",
+        capabilities=["tool_use", "vision", "thinking", "stream"],
+        input_token_price_1m=1.25,
+        output_token_price_1m=10,
+        default=True,
+    ),
     Model(
         id="gpt-4.1-mini",
         provider="openai",
@@ -158,14 +178,14 @@ _GROQ_MODELS = [
         output_token_price_1m=0.77,
     ),
     Model(
-        id="qwen-qwq-32b",
+        id="openai/gpt-oss-120b",
         provider="groq",
-        name="QwQ 32B",
-        description="SLM from Alibaba",
-        capabilities=["thinking", "tool_use"],
+        name="GPT OSS 120B",
+        description="OpenAI's flagship open-weight MoE model with 120B total parameters",
+        capabilities=["tool_use"],
         default=False,
-        input_token_price_1m=0.29,
-        output_token_price_1m=0.39,
+        input_token_price_1m=0.15,
+        output_token_price_1m=0.75,
     ),
     Model(
         id="moonshotai/kimi-k2-instruct",
@@ -257,12 +277,12 @@ _DEEPINFRA_MODELS = [
         output_token_price_1m=0.6,
     ),
     Model(
-        id="zai-org/GLM-4.5",
+        id="zai-org/GLM-4.6",
         provider="deepinfra",
-        name="Zai GLM-4.5",
-        description="The GLM-4.5 series models are foundation models designed for intelligent agents",
+        name="Zai GLM-4.6",
+        description="The GLM-4.6 series models are foundation models designed for intelligent agents",
         capabilities=["text-generation", "tool_use", "stream"],
-        input_token_price_1m=0.55,
+        input_token_price_1m=0.6,
         output_token_price_1m=2.0,
     ),
     Model(
@@ -313,9 +333,9 @@ _DEEPINFRA_MODELS = [
 ]
 _GITHUB_COPILOT_MODELS = [
     Model(
-        id="claude-3.7-sonnet",
+        id="claude-haiku-4.5",
         provider="github_copilot",
-        name="Claude 3.7 Sonnet",
+        name="Claude 4.5 Haiku",
         description="",
         capabilities=["tool_use", "vision", "stream"],
         default=False,
@@ -392,6 +412,17 @@ _GITHUB_COPILOT_MODELS = [
         input_token_price_1m=0.0,
         output_token_price_1m=0.0,
     ),
+    Model(
+        id="gpt-5-codex",
+        provider="copilot_response",
+        name="GPT 5 Codex",
+        description="",
+        capabilities=["tool_use", "vision", "stream"],
+        default=False,
+        input_token_price_1m=0.0,
+        output_token_price_1m=0.0,
+        endpoint="response",
+    ),
     Model(
         id="gpt-5-mini",
         provider="github_copilot",
@@ -432,6 +463,16 @@ _GITHUB_COPILOT_MODELS = [
         input_token_price_1m=0.0,
         output_token_price_1m=0.0,
     ),
+    Model(
+        id="claude-sonnet-4.5",
+        provider="github_copilot",
+        name="Claude Sonnet 4.5",
+        description="",
+        capabilities=["tool_use", "vision", "stream"],
+        default=False,
+        input_token_price_1m=0.0,
+        output_token_price_1m=0.0,
+    ),
     Model(
         id="claude-opus-4",
         provider="github_copilot",
