@@ -1,12 +1,15 @@
-from AgentCrew.modules import logger
+from loguru import logger
 from typing import Dict, Any, List, Optional, Callable
 from mcp import ClientSession, StdioServerParameters
 from mcp.types import Prompt, ContentBlock, TextContent, ImageContent, Tool
 from mcp.client.stdio import stdio_client
 from mcp.client.streamable_http import streamablehttp_client
+from mcp.client.sse import sse_client
 from AgentCrew.modules.agents import AgentManager, LocalAgent
 from AgentCrew.modules.tools.registry import ToolRegistry
 from .config import MCPServerConfig
+from .auth import OAuthClientResolver, FileTokenStorage
+import random
 import asyncio
 import threading
 from AgentCrew.modules import FileLogIO
@@ -27,6 +30,7 @@ class MCPService:
         self._server_connection_tasks: Dict[str, asyncio.Task] = {}
         self._server_shutdown_events: Dict[str, asyncio.Event] = {}
         self.server_prompts: Dict[str, List[Prompt]] = {}
+        self.tokens_storage_cache: Dict[str, FileTokenStorage] = {}
 
     async def _manage_single_connection(
         self, server_config: MCPServerConfig, agent_name: Optional[str] = None
@@ -45,20 +49,45 @@ class MCPService:
                 logger.info(
                     f"MCPService: Using streaming HTTP client for {server_name}"
                 )
-
                 # Prepare headers for the streamable HTTP client
                 headers = server_config.headers if server_config.headers else {}
 
-                async with streamablehttp_client(
-                    server_config.url, headers=headers
-                ) as (
-                    read_stream,
-                    write_stream,
-                    _,
-                ):
+                # Backward compatible with SSE
+                # Get or create token storage for this specific server
+                token_storage = self._get_or_create_token_storage(server_name)
+                client_info = await token_storage.get_client_info()
+                if client_info and client_info.redirect_uris:
+                    port = client_info.redirect_uris[0].port
+                else:
+                    port = random.randint(14100, 14200)
+
+                oauth_resolver = OAuthClientResolver(port=port)
+
+                if server_config.url.endswith("/sse"):
+                    session_context = sse_client(
+                        server_config.url,
+                        headers=headers,
+                        auth=oauth_resolver.get_oauth_client_provider(
+                            server_config.url, token_storage
+                        ),
+                        sse_read_timeout=60 * 60 * 24,
+                    )
+                else:
+                    session_context = streamablehttp_client(
+                        server_config.url,
+                        headers=headers,
+                        auth=oauth_resolver.get_oauth_client_provider(
+                            server_config.url, token_storage
+                        ),
+                        sse_read_timeout=60 * 60 * 24,
+                    )
+
+                async with session_context as stream_context:
                     logger.info(
                         f"MCPService: streamablehttp_client established for {server_name}"
                     )
+                    read_stream = stream_context[0]
+                    write_stream = stream_context[1]
                     async with ClientSession(read_stream, write_stream) as session:
                         logger.info(
                             f"MCPService: ClientSession established for {server_name}"
@@ -109,7 +138,20 @@ class MCPService:
                         logger.info(
                             f"MCPService: ClientSession established for {server_name}"
                         )
-                        server_info = await session.initialize()
+                        retried = 0
+                        server_info = None
+                        while retried < 3:
+                            try:
+                                server_info = await session.initialize()
+                                break
+                            except Exception:
+                                print("Retrying MCP connection...")
+                                await asyncio.sleep(retried * 2)
+                                retried += 1
+
+                        if not server_info:
+                            raise Exception("Failed to initialize MCP session.")
+
                         self.sessions[combined_server_id] = session
                         self.connected_servers[combined_server_id] = (
                             True  # Mark as connected before tool registration
@@ -164,6 +206,23 @@ class MCPService:
         """Format server ID with optional agent name prefix."""
         return f"{agent_name}__{server_name}" if agent_name else server_name
 
+    def _get_or_create_token_storage(self, server_name: str) -> FileTokenStorage:
+        """
+        Get or create a FileTokenStorage instance for a specific server.
+
+        Args:
+            server_name: Name of the MCP server
+
+        Returns:
+            FileTokenStorage instance for the server
+        """
+        if server_name not in self.tokens_storage_cache:
+            self.tokens_storage_cache[server_name] = FileTokenStorage(server_name)
+            logger.info(
+                f"MCPService: Created new FileTokenStorage for server '{server_name}'"
+            )
+        return self.tokens_storage_cache[server_name]
+
     async def start_server_connection_management(
         self, server_config: MCPServerConfig, agent_name: Optional[str] = None
     ):
