@@ -2,25 +2,20 @@ import os
 import chromadb
 import uuid
 from chromadb.config import Settings
-import numpy as np
 import queue
 import asyncio
 from typing import List, Dict, Any, Optional
 from datetime import datetime, timedelta
 from threading import Thread, Event
-from AgentCrew.modules import logger
+from loguru import logger
 import xmltodict
 
 from AgentCrew.modules.llm.base import BaseLLMService
-
 from .base_service import BaseMemoryService
-from .voyageai_ef import VoyageEmbeddingFunction
 from AgentCrew.modules.prompts.constants import (
     SEMANTIC_EXTRACTING,
     PRE_ANALYZE_PROMPT,
-    # POST_RETRIEVE_MEMORY,
 )
-from .github_copilot_ef import GithubCopilotEmbeddingFunction
 import chromadb.utils.embedding_functions as embedding_functions
 
 # Configuration constants
@@ -75,12 +70,16 @@ class ChromaMemoryService(BaseMemoryService):
 
         # Create or get collection for storing memories
         if os.getenv("VOYAGE_API_KEY"):
+            from .voyageai_ef import VoyageEmbeddingFunction
+
             voyage_ef = VoyageEmbeddingFunction(
                 api_key=os.getenv("VOYAGE_API_KEY"),
                 model_name="voyage-3.5",
             )
             self.embedding_function = voyage_ef
         elif os.getenv("GITHUB_COPILOT_API_KEY"):
+            from .github_copilot_ef import GithubCopilotEmbeddingFunction
+
             github_copilot_ef = GithubCopilotEmbeddingFunction(
                 api_key=os.getenv("GITHUB_COPILOT_API_KEY"),
                 model_name="text-embedding-3-small",
@@ -95,7 +94,8 @@ class ChromaMemoryService(BaseMemoryService):
             self.embedding_function = embedding_functions.DefaultEmbeddingFunction()
 
         self.collection = self.client.get_or_create_collection(
-            name=collection_name, embedding_function=self.embedding_function
+            name=collection_name,
+            embedding_function=self.embedding_function,  # type:ignore
         )
         # Configuration for chunking
         self.chunk_size = DEFAULT_CHUNK_SIZE
@@ -248,17 +248,22 @@ class ChromaMemoryService(BaseMemoryService):
                         .replace("{user_message}", user_message)
                         .replace("{assistant_response}", assistant_response)
                     )
-                    start_xml = analyzed_text.find("<MEMORY>")
-                    end_xml = analyzed_text.find("</MEMORY>")
-                    if start_xml != -1 and end_xml != -1:
-                        xml_content = analyzed_text[
-                            start_xml : end_xml + len("</MEMORY>")
-                        ]
-                        memory_data = xmltodict.parse(xml_content)
+                    start_xml = analyzed_text.index("<MEMORY>")
+                    end_xml = analyzed_text.index("</MEMORY>")
+                    xml_content = analyzed_text[start_xml : end_xml + len("</MEMORY>")]
+                    memory_data = xmltodict.parse(xml_content)
                     if "MEMORY" in memory_data and "ID" in memory_data["MEMORY"]:
                         ids.append(memory_data["MEMORY"]["ID"])
-                    memory_data["MEMORY"]["USER_REQUEST"] = user_message
-                    memory_data["MEMORY"]["ASSISTANT_RESPONSE"] = assistant_response
+                    if (
+                        "MEMORY" in memory_data
+                        and "USER_REQUEST" not in memory_data["MEMORY"]
+                    ):
+                        memory_data["MEMORY"]["USER_REQUEST"] = user_message
+                    if (
+                        "MEMORY" in memory_data
+                        and "ASSISTANT_RESPONSE" not in memory_data["MEMORY"]
+                    ):
+                        memory_data["MEMORY"]["ASSISTANT_RESPONSE"] = assistant_response
 
                 except Exception as e:
                     logger.warning(f"Error processing conversation with LLM: {e}")
@@ -267,7 +272,9 @@ class ChromaMemoryService(BaseMemoryService):
                         "MEMORY": {
                             "DATE": datetime.today().strftime("%Y-%m-%d"),
                             "USER_REQUEST": user_message,
-                            "ASSISTANT_RESPONSE": assistant_response,
+                            "ASSISTANT_RESPONSE": assistant_response
+                            if len(assistant_response) < 200
+                            else assistant_response[:197] + "...",
                         }
                     }
             else:
@@ -276,13 +283,15 @@ class ChromaMemoryService(BaseMemoryService):
                     "MEMORY": {
                         "DATE": datetime.today().strftime("%Y-%m-%d"),
                         "USER_REQUEST": user_message,
-                        "ASSISTANT_RESPONSE": assistant_response,
+                        "ASSISTANT_RESPONSE": assistant_response
+                        if len(assistant_response) < 200
+                        else assistant_response[:197] + "...",
                     }
                 }
 
             # Store in ChromaDB (existing logic)
             memory_id = str(uuid.uuid4())
-            timestamp = datetime.now().isoformat()
+            timestamp = datetime.now().timestamp()
             conversation_document = xmltodict.unparse(memory_data, pretty=True)
             self.current_conversation_context[session_id] = conversation_document
 
@@ -291,35 +300,27 @@ class ChromaMemoryService(BaseMemoryService):
             if len(self.context_embedding) > 5:
                 self.context_embedding.pop(0)
 
+            metadata = {
+                "date": timestamp,
+                "conversation_id": memory_id,
+                "session_id": session_id,
+                "agent": agent_name,
+                "type": "conversation",
+            }
+
             # Add to ChromaDB collection (existing logic)
             if ids:
                 self.collection.upsert(
                     ids=[ids[0]],
                     documents=[conversation_document],
                     embeddings=conversation_embedding,
-                    metadatas=[
-                        {
-                            "timestamp": timestamp,
-                            "conversation_id": memory_id,
-                            "session_id": session_id,
-                            "agent": agent_name,
-                            "type": "conversation",
-                        }
-                    ],
+                    metadatas=[metadata],
                 )
             else:
                 self.collection.add(
                     documents=[conversation_document],
                     embeddings=conversation_embedding,
-                    metadatas=[
-                        {
-                            "timestamp": timestamp,
-                            "conversation_id": memory_id,
-                            "session_id": session_id,
-                            "agent": agent_name,
-                            "type": "conversation",
-                        }
-                    ],
+                    metadatas=[metadata],
                     ids=[memory_id],
                 )
 
@@ -331,6 +332,8 @@ class ChromaMemoryService(BaseMemoryService):
             )
 
     async def need_generate_user_context(self, user_input: str) -> bool:
+        import numpy as np
+
         keywords = await self._semantic_extracting(user_input)
         if not self.loaded_conversation and self.current_embedding_context is None:
             self.current_embedding_context = self.embedding_function([keywords])
@@ -361,7 +364,7 @@ class ChromaMemoryService(BaseMemoryService):
         Returns:
             Formatted string containing relevant context from past conversations
         """
-        return self.retrieve_memory(user_input, 3, agent_name=agent_name)
+        return self.retrieve_memory(user_input, agent_name=agent_name)
 
     async def _semantic_extracting(self, input: str) -> str:
         if self.llm_service:
@@ -377,14 +380,19 @@ class ChromaMemoryService(BaseMemoryService):
             return input
 
     def retrieve_memory(
-        self, keywords: str, limit: int = 5, agent_name: str = ""
+        self,
+        keywords: str,
+        from_date: Optional[int] = None,
+        to_date: Optional[int] = None,
+        agent_name: str = "",
     ) -> str:
         """
         Retrieve relevant memories based on keywords.
 
         Args:
             keywords: Keywords to search for
-            limit: Maximum number of results to return
+            from_date: Optional start date (timestamp) to filter memories
+            to_date: Optional end date (timestamp) to filter memories
 
         Returns:
             Formatted string of relevant memories
@@ -397,9 +405,14 @@ class ChromaMemoryService(BaseMemoryService):
         if agent_name.strip():
             and_conditions.append({"agent": agent_name})
 
+        if from_date:
+            and_conditions.append({"date": {"$gte": from_date}})
+        if to_date:
+            and_conditions.append({"date": {"$lte": to_date}})
+
         results = self.collection.query(
             query_texts=[keywords],
-            n_results=limit,
+            n_results=10,
             where={"$and": and_conditions}
             if len(and_conditions) >= 2
             else and_conditions[0]
@@ -413,13 +426,14 @@ class ChromaMemoryService(BaseMemoryService):
         # Group chunks by conversation_id
         conversation_chunks = {}
         for i, (doc, metadata) in enumerate(
-            zip(results["documents"][0], results["metadatas"][0])
+            zip(results["documents"][0], results["metadatas"][0])  # type:ignore
         ):
             conv_id = metadata.get("conversation_id", "unknown")
             if conv_id not in conversation_chunks:
                 conversation_chunks[conv_id] = {
                     "chunks": [],
-                    "timestamp": metadata.get("timestamp", "unknown"),
+                    "timestamp": metadata.get("date", None)
+                    or metadata.get("timestamp", "unknown"),
                     "relevance": results["distances"][0][i]
                     if results["distances"]
                     else 99,
@@ -445,13 +459,16 @@ class ChromaMemoryService(BaseMemoryService):
             timestamp = "Unknown time"
             if conv_data["timestamp"] != "unknown":
                 try:
-                    dt = datetime.fromisoformat(conv_data["timestamp"])
+                    if isinstance(conv_data["timestamp"], int):
+                        dt = datetime.fromtimestamp(conv_data["timestamp"])
+                    else:
+                        dt = datetime.fromisoformat(conv_data["timestamp"])
                     timestamp = dt.strftime("%Y-%m-%d %H:%M")
                 except Exception:
                     timestamp = conv_data["timestamp"]
 
             output.append(
-                f"--- Memory from {timestamp} (Distance point: {conv_data['relevance']}) ---\n{conversation_text}\n---"
+                f"--- Memory from {timestamp} (relevance point(lower is better): {conv_data['relevance']}) ---\n{conversation_text}\n---"
             )
 
         memories = "\n\n".join(output)
@@ -471,6 +488,7 @@ class ChromaMemoryService(BaseMemoryService):
 
     def _cosine_similarity(self, vec_a, vec_b):
         """Calculate cosine similarity between vectors"""
+        import numpy as np
 
         a = np.array(vec_a, dtype=np.float32)
         b = np.array(vec_b, dtype=np.float32)
@@ -523,7 +541,13 @@ class ChromaMemoryService(BaseMemoryService):
 
         return len(ids_to_remove)
 
-    def forget_topic(self, topic: str, agent_name: str = "None") -> Dict[str, Any]:
+    def forget_topic(
+        self,
+        topic: str,
+        from_date: Optional[int] = None,
+        to_date: Optional[int] = None,
+        agent_name: str = "None",
+    ) -> Dict[str, Any]:
         """
         Remove memories related to a specific topic based on keyword search.
 
@@ -535,8 +559,23 @@ class ChromaMemoryService(BaseMemoryService):
         """
         try:
             # Query for memories related to the topic
+            and_conditions: List[Dict[str, Any]] = []
+
+            if agent_name.strip():
+                and_conditions.append({"agent": agent_name})
+
+            if from_date:
+                and_conditions.append({"date": {"$gte": from_date}})
+            if to_date:
+                and_conditions.append({"date": {"$lte": to_date}})
             results = self.collection.query(
-                query_texts=[topic], n_results=100, where={"agent": agent_name}
+                query_texts=[topic],
+                n_results=100,
+                where={"$and": and_conditions}
+                if len(and_conditions) >= 2
+                else and_conditions[0]
+                if and_conditions
+                else None,
             )
 
             if not results["documents"] or not results["documents"][0]:
