@@ -3,7 +3,7 @@ import os
 import uuid
 import datetime
 from typing import Dict, Any, List, Optional
-from AgentCrew.modules import logger
+from loguru import logger
 
 
 class ContextPersistenceService:
@@ -39,21 +39,12 @@ class ContextPersistenceService:
 
         if persistence_dir_override:
             persistence_dir = persistence_dir_override
-            # print(
-            #     f"INFO: Using provided persistence directory override: {persistence_dir}"
-            # )
         else:
-            env_dir = os.getenv("PERSISTENCE_DIR")
+            env_dir = os.getenv("AGENTCREW_PERSISTENCE_DIR")
             if env_dir:
                 persistence_dir = env_dir
-                # print(
-                #     f"INFO: Using persistence directory from PERSISTENCE_DIR environment variable: {persistence_dir}"
-                # )
             else:
                 persistence_dir = "./persistents"  # Default to current directory
-                # print(
-                #     "INFO: PERSISTENCE_DIR environment variable not set. Defaulting persistence directory to current directory ('.')"
-                # )
 
         # Expand user path (~) if present, and get absolute path for clarity
         self.base_dir = os.path.abspath(os.path.expanduser(persistence_dir))
@@ -421,7 +412,7 @@ class ContextPersistenceService:
 
         behavior_lower = behavior.lower().strip()
         if not behavior_lower.startswith("when"):
-            raise ValueError("Behavior must follow 'when...do...' format")
+            raise ValueError("Behavior must follow 'when..., [action]...' format")
 
         adaptive_data = self._read_json_file(
             self.adaptive_behaviors_file_path, default_value={}
