@@ -6,7 +6,7 @@ from openai import AsyncOpenAI
 from dotenv import load_dotenv
 from AgentCrew.modules.llm.base import BaseLLMService, read_binary_file, read_text_file
 from AgentCrew.modules.llm.model_registry import ModelRegistry
-from AgentCrew.modules import logger
+from loguru import logger
 
 
 class OpenAIService(BaseLLMService):
@@ -64,34 +64,45 @@ class OpenAIService(BaseLLMService):
             return input_cost + output_cost
         return 0.0
 
+    def _convert_internal_format(self, messages: List[Dict[str, Any]]):
+        for msg in messages:
+            msg.pop("agent", None)
+            msg.pop("tool_name", None)
+            if "tool_calls" in msg and msg.get("tool_calls", []):
+                for tool_call in msg["tool_calls"]:
+                    tool_call["function"] = {}
+                    tool_call["function"]["name"] = tool_call.pop("name", "")
+                    tool_call["function"]["arguments"] = json.dumps(
+                        tool_call.pop("arguments", {})
+                    )
+
+        return messages
+
     async def process_message(self, prompt: str, temperature: float = 0) -> str:
-        try:
-            response = await self.client.chat.completions.create(
-                model=self.model,
-                max_tokens=3000,
-                temperature=temperature,
-                messages=[
-                    {
-                        "role": "user",
-                        "content": prompt,
-                    }
-                ],
-            )
+        response = await self.client.chat.completions.create(
+            model=self.model,
+            max_tokens=3000,
+            temperature=temperature,
+            messages=[
+                {
+                    "role": "user",
+                    "content": prompt,
+                }
+            ],
+        )
 
-            # Calculate and log token usage and cost
-            input_tokens = response.usage.prompt_tokens if response.usage else 0
-            output_tokens = response.usage.completion_tokens if response.usage else 0
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate and log token usage and cost
+        input_tokens = response.usage.prompt_tokens if response.usage else 0
+        output_tokens = response.usage.completion_tokens if response.usage else 0
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nToken Usage Statistics:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        logger.info("\nToken Usage Statistics:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            return response.choices[0].message.content or ""
-        except Exception as e:
-            raise Exception(f"Failed to process content: {str(e)}")
+        return response.choices[0].message.content or ""
 
     def _process_file(self, file_path):
         mime_type, _ = mimetypes.guess_type(file_path)
@@ -180,6 +191,7 @@ class OpenAIService(BaseLLMService):
         stream_params = {
             "model": self.model,
             "messages": messages,
+            "parallel_tool_calls": False,
             "stream_options": {"include_usage": True},
             "max_tokens": 20000,
         }
@@ -195,9 +207,9 @@ class OpenAIService(BaseLLMService):
 
         # Add system message if provided
         if self.system_prompt:
-            stream_params["messages"] = [
-                {"role": "system", "content": self.system_prompt}
-            ] + messages
+            stream_params["messages"] = self._convert_internal_format(
+                [{"role": "system", "content": self.system_prompt}] + messages
+            )
 
         # Add tools if available
         if self.tools and "tool_use" in ModelRegistry.get_model_capabilities(
@@ -328,83 +340,6 @@ class OpenAIService(BaseLLMService):
             thinking_content,
         )
 
-    def format_tool_result(
-        self, tool_use: Dict, tool_result: Any, is_error: bool = False
-    ) -> Dict[str, Any]:
-        """
-        Format a tool result for OpenAI API.
-
-        Args:
-            tool_use: The tool use details
-            tool_result: The result from the tool execution
-            is_error: Whether the result is an error
-
-        Returns:
-            A formatted message for tool response
-        """
-        # OpenAI format for tool responses
-        message = {
-            "role": "tool",
-            "tool_call_id": tool_use["id"],
-            "content": tool_result,
-        }
-
-        # Add error indication if needed
-        if is_error:
-            message["content"] = f"ERROR: {str(message['content'])}"
-
-        return message
-
-    def format_assistant_message(
-        self, assistant_response: str, tool_uses: Optional[List[Dict]] = None
-    ) -> Dict[str, Any]:
-        """
-        Format the assistant's response for OpenAI API.
-
-        Args:
-            assistant_response: The response text
-            tool_uses: List of tool use details
-
-        Returns:
-            Formatted assistant message
-        """
-        if tool_uses and any(tu.get("id") for tu in tool_uses):
-            return {
-                "role": "assistant",
-                "content": assistant_response,
-                "tool_calls": [
-                    {
-                        "id": tool_use["id"],
-                        "function": {
-                            "name": tool_use["name"],
-                            "arguments": json.dumps(tool_use["input"]),
-                        },
-                        "type": tool_use["type"],
-                    }
-                    for tool_use in tool_uses
-                    if tool_use.get("id")  # Only include tool calls with valid IDs
-                ],
-            }
-        else:
-            return {
-                "role": "assistant",
-                "content": assistant_response,
-            }
-
-    def format_thinking_message(self, thinking_data) -> Optional[Dict[str, Any]]:
-        """
-        Format thinking content into the appropriate message format for OpenAI.
-
-        Args:
-            thinking_data: Tuple containing (thinking_content, thinking_signature)
-                or None if no thinking data is available
-
-        Returns:
-            Dict[str, Any]: A properly formatted message containing thinking blocks
-        """
-        # OpenAI doesn't support thinking blocks, so we return None
-        return None
-
     async def validate_spec(self, prompt: str) -> str:
         """
         Validate a specification prompt using OpenAI.
@@ -416,32 +351,29 @@ class OpenAIService(BaseLLMService):
             Validation result as a JSON string
         """
 
-        try:
-            response = await self.client.chat.completions.create(
-                model=self.model,
-                messages=[
-                    {
-                        "role": "user",
-                        "content": prompt,
-                    }
-                ],
-                response_format={"type": "json_object"},
-            )
+        response = await self.client.chat.completions.create(
+            model=self.model,
+            messages=[
+                {
+                    "role": "user",
+                    "content": prompt,
+                }
+            ],
+            response_format={"type": "json_object"},
+        )
 
-            # Calculate and log token usage and cost
-            input_tokens = response.usage.prompt_tokens if response.usage else 0
-            output_tokens = response.usage.completion_tokens if response.usage else 0
-            total_cost = self.calculate_cost(input_tokens, output_tokens)
+        # Calculate and log token usage and cost
+        input_tokens = response.usage.prompt_tokens if response.usage else 0
+        output_tokens = response.usage.completion_tokens if response.usage else 0
+        total_cost = self.calculate_cost(input_tokens, output_tokens)
 
-            logger.info("\nSpec Validation Token Usage:")
-            logger.info(f"Input tokens: {input_tokens:,}")
-            logger.info(f"Output tokens: {output_tokens:,}")
-            logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
-            logger.info(f"Estimated cost: ${total_cost:.4f}")
+        logger.info("\nSpec Validation Token Usage:")
+        logger.info(f"Input tokens: {input_tokens:,}")
+        logger.info(f"Output tokens: {output_tokens:,}")
+        logger.info(f"Total tokens: {input_tokens + output_tokens:,}")
+        logger.info(f"Estimated cost: ${total_cost:.4f}")
 
-            return response.choices[0].message.content or ""
-        except Exception as e:
-            raise Exception(f"Failed to validate specification: {str(e)}")
+        return response.choices[0].message.content or ""
 
     def set_system_prompt(self, system_prompt: str):
         """
